from typing import Annotated

from rapid_api_client.annotations import Path, PydanticBody

from remnawave.enums import TemplateType
from remnawave.models import GetTemplateResponseDto, UpdateTemplateRequestDto, UpdateTemplateResponseDto
from remnawave.rapid import BaseController, get, put


class SubscriptionsTemplateController(BaseController):
    @get(
        "/subscription-templates/{template_type}",
        response_class=GetTemplateResponseDto,
    )
    async def get_template(
        self,
        template_type: Annotated[TemplateType, Path(description="Template type")],
    ) -> GetTemplateResponseDto:
        """Get Template"""
        ...

    @put(
        "/subscription-templates",
        response_class=UpdateTemplateResponseDto,
    )
    async def update_template(
        self,
        body: Annotated[UpdateTemplateRequestDto, PydanticBody()],
    ) -> UpdateTemplateResponseDto:
        """Update Template"""
        ...



from typing import Annotated

from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    GetSubscriptionSettingsResponseDto,
    UpdateSubscriptionSettingsRequestDto,
    UpdateSubscriptionSettingsResponseDto,
)
from remnawave.rapid import BaseController, get, patch


class SubscriptionsSettingsController(BaseController):
    @get("/subscription-settings", response_class=GetSubscriptionSettingsResponseDto)
    async def get_settings(
        self,
    ) -> GetSubscriptionSettingsResponseDto:
        """Get Subscription Settings"""
        ...

    @patch(
        "/subscription-settings",
        response_class=UpdateSubscriptionSettingsResponseDto,
    )
    async def update_settings(
        self,
        body: Annotated[UpdateSubscriptionSettingsRequestDto, PydanticBody()],
    ) -> UpdateSubscriptionSettingsResponseDto:
        """Update Subscription Settings"""
        ...



from typing import Annotated

from rapid_api_client import Query

from remnawave.models import GetNodesUsageByRangeResponseDto, GetNodesRealtimeUsageResponseDto
from remnawave.rapid import BaseController, get


class BandWidthStatsController(BaseController):
    @get("/nodes/usage/range", response_class=GetNodesUsageByRangeResponseDto)
    async def get_nodes_usage_by_range(
        self,
        start: Annotated[str, Query(description="Start date in ISO format")],
        end: Annotated[str, Query(description="End date in ISO format")],
    ) -> GetNodesUsageByRangeResponseDto:
        """Get Nodes Usage By Range"""
        ...

    @get("/nodes/usage/realtime", response_class=GetNodesRealtimeUsageResponseDto)
    async def get_nodes_usage_realtime(
        self,
    ) -> GetNodesRealtimeUsageResponseDto:
        """Get Nodes Usage Realtime"""
        ...
        


from datetime import datetime
from typing import Annotated, Dict, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, StringConstraints


class SubscriptionSettingsResponseDto(BaseModel):
    uuid: UUID
    profile_title: str = Field(alias="profileTitle")
    support_link: str = Field(alias="supportLink")
    profile_update_interval: int = Field(
        alias="profileUpdateInterval", strict=True, ge=1
    )
    is_profile_webpage_url_enabled: bool = Field(alias="isProfileWebpageUrlEnabled")
    serve_json_at_base_subscription: bool = Field(alias="serveJsonAtBaseSubscription")
    add_username_to_base_subscription: bool = Field(
        alias="addUsernameToBaseSubscription"
    )
    show_custom_remarks: bool = Field(alias="isShowCustomRemarks")
    happ_announce: Optional[str] = Field(None, alias="happAnnounce")
    happ_routing: Optional[str] = Field(None, alias="happRouting")
    expired_users_remarks: List[str] = Field(alias="expiredUsersRemarks")
    limited_users_remarks: List[str] = Field(alias="limitedUsersRemarks")
    disabled_users_remarks: List[str] = Field(alias="disabledUsersRemarks")
    custom_response_headers: Optional[Dict[str, str]] = Field(
        None, alias="customResponseHeaders"
    )
    randomize_hosts: bool = Field(alias="randomizeHosts")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class GetSubscriptionSettingsResponseDto(SubscriptionSettingsResponseDto):
    pass


class UpdateSubscriptionSettingsResponseDto(SubscriptionSettingsResponseDto):
    pass


class UpdateSubscriptionSettingsRequestDto(BaseModel):
    uuid: UUID
    profile_title: Optional[str] = Field(None, serialization_alias="profileTitle")
    support_link: Optional[str] = Field(None, serialization_alias="supportLink")
    profile_update_interval: Optional[int] = Field(
        None, serialization_alias="profileUpdateInterval"
    )
    is_profile_webpage_url_enabled: Optional[bool] = Field(
        None, serialization_alias="isProfileWebpageUrlEnabled"
    )
    serve_json_at_base_subscription: Optional[bool] = Field(
        None, serialization_alias="serveJsonAtBaseSubscription"
    )
    add_username_to_base_subscription: Optional[bool] = Field(
        None, serialization_alias="addUsernameToBaseSubscription"
    )
    is_show_custom_remarks: Optional[bool] = Field(
        None, serialization_alias="isShowCustomRemarks"
    )
    happ_announce: Annotated[Optional[str], StringConstraints(max_length=200)] = Field(
        None, serialization_alias="happAnnounce"
    )
    happ_routing: Optional[str] = Field(None, serialization_alias="happRouting")
    expired_users_remarks: Optional[List[str]] = Field(
        None, serialization_alias="expiredUsersRemarks"
    )
    limited_users_remarks: Optional[List[str]] = Field(
        None, serialization_alias="limitedUsersRemarks"
    )
    disabled_users_remarks: Optional[List[str]] = Field(
        None, serialization_alias="disabledUsersRemarks"
    )
    custom_response_headers: Optional[Dict[str, str]] = Field(
        None, serialization_alias="customResponseHeaders"
    )
    randomize_hosts: Optional[bool] = Field(None, serialization_alias="randomizeHosts")


from datetime import datetime
from typing import Annotated, List, Optional
from uuid import UUID

from pydantic import (
    BaseModel,
    Field,
    RootModel,
    StringConstraints,
)

from remnawave.enums import TrafficLimitStrategy, UserStatus


class UserActiveInboundsDto(BaseModel):
    uuid: UUID
    tag: str
    type: str
    network: str | None = None
    security: str | None = None


class UserLastConnectedNodeDto(BaseModel):
    connected_at: datetime = Field(alias="connectedAt")
    node_name: str = Field(alias="nodeName")


class ActiveInternalSquadDto(BaseModel):
    uuid: UUID
    name: str


class HappCrypto(BaseModel):
    cryptoLink: str


class CreateUserRequestDto(BaseModel):
    expire_at: datetime = Field(..., serialization_alias="expireAt")
    username: Annotated[
        str, StringConstraints(pattern=r"^[a-zA-Z0-9_-]+$", min_length=3, max_length=36)
    ]
    created_at: datetime | None = Field(None, serialization_alias="createdAt")
    status: UserStatus | None = None
    short_uuid: str | None = Field(None, serialization_alias="shortUuid")
    trojan_password: Annotated[
        str | None, StringConstraints(min_length=8, max_length=32)
    ] = Field(None, serialization_alias="trojanPassword")
    vless_uuid: str | None = Field(None, serialization_alias="vlessUuid")
    ss_password: Annotated[
        str | None, StringConstraints(min_length=8, max_length=32)
    ] = Field(None, serialization_alias="ssPassword")
    traffic_limit_bytes: int | None = Field(
        None, serialization_alias="trafficLimitBytes", strict=True, ge=0
    )
    traffic_limit_strategy: TrafficLimitStrategy | None = Field(
        None, serialization_alias="trafficLimitStrategy"
    )
    last_traffic_reset_at: datetime | None = Field(
        None, serialization_alias="lastTrafficResetAt"
    )
    description: str | None = None
    tag: str | None = None
    telegram_id: int | None = Field(None, serialization_alias="telegramId")
    email: str | None = None
    hwidDeviceLimit: int | None = Field(
        None, serialization_alias="hwidDeviceLimit", strict=True, ge=0
    )
    active_internal_squads: list[str] | None = Field(
        None, serialization_alias="activeInternalSquads"
    )
    uuid: Optional[UUID] = Field(None, description="UUID of the user. Optional. If not provided, a new UUID will be generated by Remnawave.")


class UpdateUserRequestDto(BaseModel):
    uuid: UUID
    active_internal_squads: list[str] | None = Field(
        None, serialization_alias="activeInternalSquads"
    )
    description: str | None = None
    email: str | None = None
    expire_at: datetime | None = Field(None, serialization_alias="expireAt")
    hwidDeviceLimit: int | None = Field(
        None, serialization_alias="hwidDeviceLimit", strict=True, ge=0
    )
    status: UserStatus | None = None
    tag: str | None = None
    telegram_id: int | None = Field(None, serialization_alias="telegramId")
    traffic_limit_bytes: int | None = Field(
        None, serialization_alias="trafficLimitBytes", strict=True, ge=0
    )
    traffic_limit_strategy: TrafficLimitStrategy | None = Field(
        None, serialization_alias="trafficLimitStrategy"
    )


class UserResponseDto(BaseModel):
    uuid: UUID
    short_uuid: str = Field(alias="shortUuid")
    username: str
    status: UserStatus | None = None
    used_traffic_bytes: float = Field(alias="usedTrafficBytes")
    lifetime_used_traffic_bytes: float = Field(alias="lifetimeUsedTrafficBytes")
    traffic_limit_bytes: int | None = Field(None, alias="trafficLimitBytes")
    traffic_limit_strategy: str | None = Field(None, alias="trafficLimitStrategy")
    sub_last_user_agent: str | None = Field(None, alias="subLastUserAgent")
    sub_last_opened_at: datetime | None = Field(None, alias="subLastOpenedAt")
    expire_at: datetime | None = Field(None, alias="expireAt")
    online_at: datetime | None = Field(None, alias="onlineAt")
    sub_revoked_at: datetime | None = Field(None, alias="subRevokedAt")
    last_traffic_reset_at: datetime | None = Field(None, alias="lastTrafficResetAt")
    trojan_password: str = Field(alias="trojanPassword")
    vless_uuid: UUID = Field(alias="vlessUuid")
    ss_password: str = Field(alias="ssPassword")
    description: str | None = None
    telegram_id: int | None = Field(None, alias="telegramId")
    email: str | None = None
    hwidDeviceLimit: int | None = Field(
        None, serialization_alias="hwidDeviceLimit", strict=True, ge=0
    )
    active_internal_squads: list[ActiveInternalSquadDto] | None = Field(
        None, alias="activeInternalSquads"
    )
    subscription_url: str | None = Field(None, alias="subscriptionUrl")
    first_connected: datetime | None = Field(None, alias="firstConnectedAt")
    last_trigger_threshold: int | None = Field(None, alias="lastTriggeredThreshold")
    last_connected_node: UserLastConnectedNodeDto | None = Field(
        None, alias="lastConnectedNode"
    )
    happ: HappCrypto | None = Field(None, alias="happ")
    tag: str | None = Field(None, alias="tag")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class EmailUserResponseDto(RootModel[list[UserResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class TagUserResponseDto(RootModel[list[UserResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class TelegramUserResponseDto(RootModel[list[UserResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class UsersResponseDto(BaseModel):
    users: list[UserResponseDto]
    total: float


class DeleteUserResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class TagsResponseDto(BaseModel):
    tags: list[str]


class CreateUserResponseDto(UserResponseDto):
    pass


class UpdateUserResponseDto(UserResponseDto):
    pass


class DisableUserResponseDto(UserResponseDto):
    pass


class EnableUserResponseDto(UserResponseDto):
    pass


class ResetUserTrafficResponseDto(UserResponseDto):
    pass


class RevokeUserSubscriptionResponseDto(UserResponseDto):
    pass


class ActivateAllInboundsResponseDto(UserResponseDto):
    pass


class GetUserByUuidResponseDto(UserResponseDto):
    pass


class GetUserByShortUuidResponseDto(UserResponseDto):
    pass


class GetUserByUsernameResponseDto(UserResponseDto):
    pass


class RevokeUserRequestDto(BaseModel):
    short_uuid: str | None = Field(
        None,
        serialization_alias="shortUuid",
        description="Optional. If not provided, a new short UUID will be generated by Remnawave. Please note that it is strongly recommended to allow Remnawave to generate the short UUID.",
        min_length=6,
        max_length=48,
        pattern=r"^[a-zA-Z0-9_-]+$",
    )


class SubscriptionRequestRecord(BaseModel):
    id: int
    user_uuid: UUID = Field(alias="userUuid")
    request_at: datetime = Field(alias="requestAt")
    request_ip: Optional[str] = Field(alias="requestIp")
    user_agent: Optional[str] = Field(alias="userAgent")


class SubscriptionRequestsResponseData(BaseModel):
    total: float
    records: List[SubscriptionRequestRecord]


class GetSubscriptionRequestsResponseDto(SubscriptionRequestsResponseData):
    pass


from typing import Annotated, Optional

from rapid_api_client import Path, Query
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateUserRequestDto,
    DeleteUserResponseDto,
    EmailUserResponseDto,
    GetUserAccessibleNodesResponseDto,
    TelegramUserResponseDto,
    UpdateUserRequestDto,
    UserResponseDto,
    UsersResponseDto,
    TagsResponseDto,
    TagUserResponseDto,
    RevokeUserRequestDto,
    GetSubscriptionRequestsResponseDto
)
from remnawave.rapid import BaseController, delete, get, patch, post


class UsersController(BaseController):
    @post("/users", response_class=UserResponseDto)
    async def create_user(
        self,
        body: Annotated[CreateUserRequestDto, PydanticBody()],
    ) -> UserResponseDto:
        """Create User"""
        ...

    @patch("/users", response_class=UserResponseDto)
    async def update_user(
        self,
        body: Annotated[UpdateUserRequestDto, PydanticBody()],
    ) -> UserResponseDto:
        """Update User"""
        ...

    @get("/users", response_class=UsersResponseDto)
    async def get_all_users_v2(
        self,
        start: Annotated[
            int, Query(default=0, ge=0, description="Index to start pagination from")
        ],
        size: Annotated[
            int, Query(default=25, ge=1, description="Number of users per page")
        ],
    ) -> UsersResponseDto:
        """
        Get users page from the end.

        Args:
            page (int): Page number from the end (1 = last page).
            size (int): Number of users per page.

        Returns:
            UsersResponseDto
        """
        ...

    @delete("/users/{uuid}", response_class=DeleteUserResponseDto)
    async def delete_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> DeleteUserResponseDto:
        """Delete User"""
        ...

    @post("users/{uuid}/actions/revoke", response_class=UserResponseDto)
    async def revoke_user_subscription(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
        body: Optional[Annotated[RevokeUserRequestDto, PydanticBody()]] = None,
    ) -> UserResponseDto:
        """Revoke User Subscription"""
        ...

    @post("/users/{uuid}/actions/disable", response_class=UserResponseDto)
    async def disable_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UserResponseDto:
        """Disable User"""
        ...

    @post("/users/{uuid}/actions/enable", response_class=UserResponseDto)
    async def enable_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UserResponseDto:
        """Enable User"""
        ...

    @post("/users/{uuid}/actions/reset-traffic", response_class=UserResponseDto)
    async def reset_user_traffic(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UserResponseDto:
        """Reset User Traffic"""
        ...

    @get("/users/by-short-uuid/{short_uuid}", response_class=UserResponseDto)
    async def get_user_by_short_uuid(
        self,
        short_uuid: Annotated[str, Path(description="Short UUID of the user")],
    ) -> UserResponseDto:
        """Get User By Short UUID"""
        ...

    @get(
        "/users/by-subscription-uuid/{subscription_uuid}",
        response_class=UserResponseDto,
    )
    async def get_user_by_subscription_uuid(
        self,
        subscription_uuid: Annotated[str, Path(description="UUID of the subscription")],
    ) -> UserResponseDto:
        """Get User By Subscription UUID"""
        ...

    @get("/users/{uuid}", response_class=UserResponseDto)
    async def get_user_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UserResponseDto:
        """Get User By UUID"""
        ...

    @get("/users/by-username/{username}", response_class=UserResponseDto)
    async def get_user_by_username(
        self,
        username: Annotated[str, Path(description="Username of the user")],
    ) -> UserResponseDto:
        """Get User By Username"""
        ...

    @get(
        "/users/by-telegram-id/{telegram_id}",
        response_class=TelegramUserResponseDto,
    )
    async def get_users_by_telegram_id(
        self,
        telegram_id: Annotated[str, Path(description="Telegram ID of the user")],
    ) -> TelegramUserResponseDto:
        """Get Users By Telegram ID"""
        ...

    @get("/users/by-email/{email}", response_class=EmailUserResponseDto)
    async def get_users_by_email(
        self,
        email: Annotated[str, Path(description="Email of the user")],
    ) -> EmailUserResponseDto:
        """Get Users By Email"""
        ...

    @get("/users/by-tag/{tag}", response_class=TagUserResponseDto)
    async def get_users_by_tag(
        self,
        tag: Annotated[str, Path(description="Tag of the user")],
    ) -> TagUserResponseDto:
        """Get Users By Tag"""
        ...

    @get("/users/tags", response_class=TagsResponseDto)
    async def get_all_tags(
        self,
    ) -> TagsResponseDto:
        """Get All Tags"""
        ...

    @get(
        "/users/{uuid}/accessible-nodes",
        response_class=GetUserAccessibleNodesResponseDto,
    )
    async def get_user_accessible_nodes(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> GetUserAccessibleNodesResponseDto:
        """Get User Accessible Nodes"""
        ...

    @get("/users/{uuid}/subscription-request-history", response_class=GetSubscriptionRequestsResponseDto)
    async def get_subscription_requests(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> GetSubscriptionRequestsResponseDto:
        """Get Subscription Requests History"""
        ...


from typing import Annotated, List

from rapid_api_client import Path
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateHostRequestDto,
    CreateHostResponseDto,
    DeleteHostResponseDto,
    GetAllHostsResponseDto,
    GetOneHostResponseDto,
    ReorderHostRequestDto,
    ReorderHostResponseDto,
    UpdateHostRequestDto,
    UpdateHostResponseDto,
    GetAllHostTagsResponseDto,
)
from remnawave.rapid import AttributeBody, BaseController, delete, get, post, patch


class HostsController(BaseController):
    @post("/hosts", response_class=CreateHostResponseDto)
    async def create_host(
        self,
        body: Annotated[CreateHostRequestDto, PydanticBody()],
    ) -> CreateHostResponseDto:
        """Create Host"""
        ...

    @patch("/hosts", response_class=UpdateHostResponseDto)
    async def update_host(
        self,
        body: Annotated[UpdateHostRequestDto, PydanticBody()],
    ) -> UpdateHostResponseDto:
        """Update Host"""
        ...

    @get("/hosts", response_class=GetAllHostsResponseDto)
    async def get_all_hosts(
        self,
    ) -> GetAllHostsResponseDto:
        """Get All Hosts"""
        ...

    @get("/hosts/tags", response_class=GetAllHostTagsResponseDto)
    async def get_hosts_tags(
        self,
    ) -> GetAllHostTagsResponseDto:
        """Get Hosts Tags"""
        ...

    @delete("/hosts/{uuid}", response_class=DeleteHostResponseDto)
    async def delete_host(
        self,
        uuid: Annotated[str, Path(description="UUID of the host")],
    ) -> DeleteHostResponseDto:
        """Delete Host"""
        ...

    @get("/hosts/{uuid}", response_class=GetOneHostResponseDto)
    async def get_one_host(
        self,
        uuid: Annotated[str, Path(description="UUID of the host")],
    ) -> GetOneHostResponseDto:
        """Get One Host"""
        ...

    @post("/hosts/actions/reorder", response_class=ReorderHostResponseDto)
    async def reorder_hosts(
        self,
        body: Annotated[ReorderHostRequestDto, PydanticBody()],
    ) -> ReorderHostResponseDto:
        """Reorder Hosts"""
        ...



from typing import Any, Optional
from uuid import UUID

from pydantic import BaseModel, Field

from remnawave.enums import TemplateType


class TemplateResponseDto(BaseModel):
    uuid: UUID
    template_type: TemplateType = Field(alias="templateType")
    template_json: Optional[Any] = Field(None, alias="templateJson")
    encoded_template_yaml: Optional[str] = Field(None, alias="encodedTemplateYaml")


class GetTemplateResponseDto(TemplateResponseDto):
    pass


class UpdateTemplateRequestDto(BaseModel):
    template_type: TemplateType = Field(serialization_alias="templateType")
    template_json: Optional[dict] = Field(None, serialization_alias="templateJson")
    encoded_template_yaml: Optional[str] = Field(
        None, serialization_alias="encodedTemplateYaml"
    )


class UpdateTemplateResponseDto(TemplateResponseDto):
    pass



from typing import Annotated, Optional
from uuid import UUID

from rapid_api_client import Path
from remnawave.models import GetAllInboundsResponseDto, GetInboundsByProfileUuidResponseDto
from remnawave.rapid import BaseController, get


class InboundsController(BaseController):
    @get("/config-profiles/inbounds", response_class=GetAllInboundsResponseDto)
    async def get_all_inbounds(
        self,
    ) -> GetAllInboundsResponseDto:
        """Get all inbounds from all config profiles"""
        ...

    @get("/config-profiles/{uuid}/inbounds", response_class=GetInboundsByProfileUuidResponseDto)
    async def get_inbounds_by_profile_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the config profile")],
    ) -> GetInboundsByProfileUuidResponseDto:
        """Get inbounds by profile uuid"""
        ...



from typing import Annotated

from rapid_api_client import Path, Query

from remnawave.models import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesUsageByRangeResponseDto,
)
from remnawave.rapid import BaseController, get


class NodesUsageHistoryController(BaseController):
    @get("/nodes/usage/range", response_class=GetNodesUsageByRangeResponseDto)
    async def get_nodes_usage_by_range(
        self,
        start: Annotated[str, Query(description="Start date", format="date-time")],
        end: Annotated[str, Query(description="End date", format="date-time")],
    ) -> GetNodesUsageByRangeResponseDto:
        """Get nodes usage by range"""
        ...


class NodesUserUsageHistoryController(BaseController):
    @get("/nodes/usage/{uuid}/users/range", response_class=GetNodeUserUsageByRangeResponseDto)
    async def get_node_user_usage_by_range(
        self,
        uuid: Annotated[str, Path(description="UUID of the node")],
        start: Annotated[str, Query(description="Start date", format="date-time")],
        end: Annotated[str, Query(description="End date", format="date-time")],
    ) -> GetNodeUserUsageByRangeResponseDto:
        """Get nodes user usage by range"""
        ...



from .api_tokens_management import (
    CreateApiTokenRequestDto,
    CreateApiTokenResponseDto,
    DeleteApiTokenResponseDto,
    FindAllApiTokensResponseDto,
)
from .auth import (
    GetStatusResponseDto,
    LoginRequestDto,
    LoginResponseDto,
    RegisterRequestDto,
    RegisterResponseDto,
    StatusResponseDto,  # Legacy alias
    TelegramCallbackRequestDto,
    TelegramCallbackResponseDto,
    LoginTelegramRequestDto,  # Legacy alias
)
from .bandwidthstats import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesRealtimeUsageResponseDto,
    GetNodesUsageByRangeResponseDto,
    GetUserUsageByRangeResponseDto,
    NodeRealtimeUsageResponseDto,
    NodeUsageResponseDto,
    NodesRealtimeUsageResponseDto,  # Legacy alias
    NodesUsageResponseDto,  # Legacy alias
)
from .config_profiles import (
    ConfigProfileDto,
    CreateConfigProfileRequestDto,
    CreateConfigProfileResponseDto,
    DeleteConfigProfileResponseDto,
    GetAllConfigProfilesResponseDto,
    GetAllConfigProfilesResponsePaginated,
    GetAllInboundsResponseDto,
    GetConfigProfileByUuidResponseDto,
    GetInboundsByProfileUuidResponseDto,
    InboundDto,
    UpdateConfigProfileRequestDto,
    UpdateConfigProfileResponseDto,
)
from .hosts import (
    CreateHostInboundData,
    CreateHostRequestDto,
    CreateHostResponseDto,
    DeleteHostResponseDto,
    GetAllHostsResponseDto,
    GetOneHostResponseDto,
    HostInboundData,
    HostResponseDto,
    HostsResponseDto,  # Legacy alias
    ReorderHostItem,
    ReorderHostRequestDto,
    ReorderHostResponseDto,
    UpdateHostRequestDto,
    UpdateHostResponseDto,
    GetAllHostTagsResponseDto,
)
from .hosts_bulk_actions import (
    BulkDeleteHostsResponseDto,
    BulkDisableHostsResponseDto,
    BulkEnableHostsResponseDto,
    SetInboundToManyHostsRequestDto,
    SetInboundToManyHostsResponseDto,
    SetPortToManyHostsResponseDto,
    SetPortToManyHostsRequestDto
)
from .hwid import (
    CreateHWIDUser,  # Legacy alias
    CreateUserHwidDeviceRequestDto,
    CreateUserHwidDeviceResponseDto,
    DeleteUserHwidDeviceRequestDto,
    DeleteUserHwidDeviceResponseDto,
    GetUserHwidDevicesResponseDto,
    HWIDDeleteRequest,  # Legacy alias
    HWIDUserResponseDto,  # Legacy alias
    HWIDUserResponseDtoList,  # Legacy alias
    GetHwidStatisticsResponseDto,
    DeleteUserAllHwidDeviceRequestDto
)
from .inbounds import (
    AllInboundsData,
    FullInboundResponseDto,
    FullInboundStatistic,
    FullInboundsResponseDto,
    GetAllInboundsResponseDto,
    GetFullInboundsResponseDto,
    GetInboundsByProfileUuidResponseDto,
    GetInboundsResponseDto,
    InboundResponseDto,
    InboundsByProfileData,
    InboundsResponseDto,  # Legacy alias
)
from .inbounds_bulk_actions import (
    AddInboundToNodesResponseDto,
    AddInboundToUsersResponseDto,
    RemoveInboundFromNodesResponseDto,
    RemoveInboundFromUsersResponseDto,
)
from .infra_billing import (
    CreateInfraBillingNodeRequestDto,
    CreateInfraBillingNodeResponseDto,
    CreateInfraProviderRequestDto,
    CreateInfraProviderResponseDto,
    DeleteInfraBillingNodeResponseDto,
    DeleteInfraProviderResponseDto,
    GetAllInfraBillingHistoryResponseDto,
    GetAllInfraBillingNodesResponseDto,
    GetAllInfraProvidersResponseDto,
    GetInfraBillingHistoryByUuidResponseDto,
    GetInfraBillingNodeByUuidResponseDto,
    GetInfraProviderByUuidResponseDto,
    InfraBillingHistoryDto,
    InfraBillingNodeDto,
    InfraProviderDto,
    NodeDto,
    UpdateInfraBillingNodeRequestDto,
    UpdateInfraBillingNodeResponseDto,
    UpdateInfraProviderRequestDto,
    UpdateInfraProviderResponseDto,
)
from .internal_squads import (
    AddUsersToInternalSquadRequestDto,
    AddUsersToInternalSquadResponseDto,
    CreateInternalSquadRequestDto,
    CreateInternalSquadResponseDto,
    DeleteInternalSquadResponseDto,
    DeleteUsersFromInternalSquadRequestDto,
    DeleteUsersFromInternalSquadResponseDto,
    GetAllInternalSquadsResponseDto,
    GetInternalSquadByUuidResponseDto,
    InternalSquadDto,
    UpdateInternalSquadRequestDto,
    UpdateInternalSquadResponseDto,
    GetInternalSquadAccessibleNodesResponseDto,
)
from .keygen import GetPubKeyResponseDto, PubKeyResponseDto  # Legacy alias
from .nodes import (
    CreateNodeRequestDto,
    CreateNodeResponseDto,
    DeleteNodeResponseDto,
    DisableNodeResponseDto,
    EnableNodeResponseDto,
    ExcludedInbounds,
    GetAllNodesResponseDto,
    GetOneNodeResponseDto,
    NodeConfigProfileDto,
    NodeConfigProfileRequestDto,
    NodeResponseDto,
    NodesResponseDto,  # Legacy alias
    ReorderNodeRequestDto,
    ReorderNodeResponseDto,
    RestartAllNodesResponseDto,
    RestartNodeResponseDto,
    UpdateNodeRequestDto,
    UpdateNodeResponseDto,
    RestartAllNodesRequestDto,
)
from .nodes_usage_history import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesUsageByRangeResponseDto,
    GetUserAccessibleNodesResponseDto,
    NodeInfoDto,
    NodeUsageDto,
    UserUsageDto,
)
from .subscription import (
    GetAllSubscriptionsResponseDto,
    GetSubscriptionByUsernameResponseDto,
    GetSubscriptionInfoResponseDto,
    SubscriptionInfoResponseDto,  # Legacy alias
    UserSubscription,
    GetRawSubscriptionByShortUuidResponseDto,
    GetSubscriptionByShortUUIDResponseDto,
    GetSubscriptionByUUIDResponseDto,
)
from .subscriptions_settings import (
    GetSubscriptionSettingsResponseDto,
    SubscriptionSettingsResponseDto,
    UpdateSubscriptionSettingsRequestDto,
    UpdateSubscriptionSettingsResponseDto,
)
from .subscriptions_template import (
    GetTemplateResponseDto,
    TemplateResponseDto,
    UpdateTemplateRequestDto,
    UpdateTemplateResponseDto,
)
from .system import (
    BandwidthStatistic,
    BandwidthStatisticResponseDto,
    CPUStatistic,
    GetBandwidthStatsResponseDto,
    GetNodesStatisticsResponseDto,
    GetRemnawaveHealthResponseDto,
    GetStatsResponseDto,
    MemoryStatistic,
    NodeStatistic,
    NodesStatisticResponseDto,
    OnlineStatistic,
    StatisticResponseDto,
    StatusCounts,
    UsersStatistic,
    GetNodesMetricsResponseDto,
    GetX25519KeyPairResponseDto, 
    X25519KeyPair,
)
from .users import (
    ActiveInternalSquadDto,
    CreateUserRequestDto,
    DeleteUserResponseDto,
    EmailUserResponseDto,
    TelegramUserResponseDto,
    UpdateUserRequestDto,
    UserActiveInboundsDto,
    UserLastConnectedNodeDto,
    UserResponseDto,
    UsersResponseDto,
    TagsResponseDto,
    TagUserResponseDto,
    RevokeUserRequestDto,
    GetSubscriptionRequestsResponseDto,
)
from .users_bulk_actions import (
    BulkAllResetTrafficUsersResponseDto,
    BulkAllUpdateUsersRequestDto,
    BulkAllUpdateUsersResponseDto,
    BulkResponseDto,
    BulkUpdateUsersInternalSquadsRequestDto,
    UpdateUserFields,
)
from .users_stats import UserUsageByRange, UserUsageByRangeResponseDto
from .xray_config import (
    ConfigResponseDto,  # Legacy alias
    GetConfigResponseDto,
    UpdateConfigRequestDto,
    UpdateConfigResponseDto,
)
from .subscription_request_history import (
    GetAllSubscriptionRequestHistoryResponseDto,
    GetSubscriptionRequestHistoryStatsResponseDto,
    SubscriptionRequestHistoryRecord,
    SubscriptionRequestHistoryData,
    AppStatItem,
    HourlyRequestStat,
    SubscriptionRequestHistoryStatsData
)
from .webhook import (
    UserEventDto, 
    UserHwidDeviceEventDto,
    HwidUserDeviceDto,
    LastConnectedNodeDto,
    InternalSquadDto,
    BaseUserDto,
    UserDto,
    NodeDto,
    ConfigProfileInboundDto,
    InfraProviderDto,
    LoginAttemptDto,
    ServiceEventDto,
    NodeEventDto,
    CustomErrorEventDto,
    CrmEventDto,
    WebhookPayloadDto,
)

__all__ = [
    # Auth models
    "GetStatusResponseDto",
    "LoginRequestDto",
    "LoginResponseDto",
    "RegisterRequestDto",
    "RegisterResponseDto",
    "StatusResponseDto",  # Legacy alias
    "TelegramCallbackRequestDto",
    "TelegramCallbackResponseDto",
    "LoginTelegramRequestDto",  # Legacy alias
    # Nodes models
    "CreateNodeRequestDto",
    "CreateNodeResponseDto",
    "DeleteNodeResponseDto",
    "DisableNodeResponseDto",
    "EnableNodeResponseDto",
    "ExcludedInbounds",
    "GetAllNodesResponseDto",
    "GetOneNodeResponseDto",
    "NodeResponseDto",
    "NodesResponseDto",  # Legacy alias
    "ReorderNodeRequestDto",
    "ReorderNodeResponseDto",
    "RestartAllNodesResponseDto",
    "RestartNodeResponseDto",
    "UpdateNodeRequestDto",
    "UpdateNodeResponseDto",
    "NodeConfigProfileDto",
    "NodeConfigProfileRequestDto",
    "RestartAllNodesRequestDto",
    # Hosts models
    "CreateHostRequestDto",
    "CreateHostResponseDto",
    "DeleteHostResponseDto",
    "GetAllHostsResponseDto",
    "GetOneHostResponseDto",
    "HostResponseDto",
    "HostsResponseDto",  # Legacy alias
    "ReorderHostRequestDto",
    "ReorderHostResponseDto",
    "UpdateHostRequestDto",
    "UpdateHostResponseDto",
    "GetAllHostTagsResponseDto",
    "CreateHostInboundData",
    "HostInboundData",
    "ReorderHostItem",
    # Inbounds models
    "AllInboundsData",
    "FullInboundResponseDto",
    "FullInboundStatistic",
    "FullInboundsResponseDto",
    "GetFullInboundsResponseDto",
    "GetInboundsResponseDto",
    "InboundResponseDto",
    "InboundsResponseDto",  # Legacy alias
    "GetInternalSquadAccessibleNodesResponseDto",
    "InboundsByProfileData",
    # Keygen models
    "GetPubKeyResponseDto",
    "PubKeyResponseDto",  # Legacy alias
    # Subscription models
    "GetAllSubscriptionsResponseDto",
    "GetSubscriptionByUsernameResponseDto",
    "GetSubscriptionByShortUUIDResponseDto",
    "GetSubscriptionByUUIDResponseDto",
    "GetSubscriptionInfoResponseDto",
    "SubscriptionInfoResponseDto",  # Legacy alias
    "UserSubscription",
    "GetRawSubscriptionByShortUuidResponseDto",
    # Subscription settings models
    "GetSubscriptionSettingsResponseDto",
    "SubscriptionSettingsResponseDto",
    "UpdateSubscriptionSettingsRequestDto",
    "UpdateSubscriptionSettingsResponseDto",
    # Subscription template models
    "GetTemplateResponseDto",
    "TemplateResponseDto",
    "UpdateTemplateRequestDto",
    "UpdateTemplateResponseDto",
    # System models
    "BandwidthStatistic",
    "BandwidthStatisticResponseDto",
    "CPUStatistic",
    "GetBandwidthStatsResponseDto",
    "GetNodesStatisticsResponseDto",
    "GetRemnawaveHealthResponseDto",
    "GetStatsResponseDto",
    "MemoryStatistic",
    "NodeStatistic",
    "NodesStatisticResponseDto",
    "OnlineStatistic",
    "StatisticResponseDto",
    "StatusCounts",
    "UsersStatistic",
    "GetNodesMetricsResponseDto",
    "GetX25519KeyPairResponseDto",
    "X25519KeyPair",
    # XRay config models
    "ConfigResponseDto",  # Legacy alias
    "GetConfigResponseDto",
    "UpdateConfigRequestDto",
    "UpdateConfigResponseDto",
    # HWID models
    "CreateHWIDUser",  # Legacy alias
    "CreateUserHwidDeviceRequestDto",
    "CreateUserHwidDeviceResponseDto",
    "DeleteUserHwidDeviceRequestDto",
    "DeleteUserHwidDeviceResponseDto",
    "GetUserHwidDevicesResponseDto",
    "HWIDDeleteRequest",  # Legacy alias
    "HWIDUserResponseDto",  # Legacy alias
    "HWIDUserResponseDtoList",  # Legacy alias
    "GetHwidStatisticsResponseDto",
    "DeleteUserAllHwidDeviceRequestDto",
    # Bandwidth stats models
    "GetNodeUserUsageByRangeResponseDto",
    "GetNodesRealtimeUsageResponseDto",
    "GetNodesUsageByRangeResponseDto",
    "GetUserUsageByRangeResponseDto",
    "NodeRealtimeUsageResponseDto",
    "NodeUsageResponseDto",
    "NodesRealtimeUsageResponseDto",  # Legacy alias
    "NodesUsageResponseDto",  # Legacy alias
    # API Tokens models
    "CreateApiTokenRequestDto",
    "CreateApiTokenResponseDto",
    "DeleteApiTokenResponseDto",
    "FindAllApiTokensResponseDto",
    # Inbound bulk actions models
    "AddInboundToNodesResponseDto",
    "AddInboundToUsersResponseDto",
    "RemoveInboundFromNodesResponseDto",
    "RemoveInboundFromUsersResponseDto",
    # Host bulk actions models
    "BulkDeleteHostsResponseDto",
    "BulkDisableHostsResponseDto",
    "BulkEnableHostsResponseDto",
    "SetInboundToManyHostsRequestDto",
    "SetInboundToManyHostsResponseDto",
    "SetPortToManyHostsResponseDto",
    "SetPortToManyHostsRequestDto",
    # Users models
    "ActiveInternalSquadDto",
    "CreateUserRequestDto",
    "CreateUserResponseDto",
    "DeleteUserResponseDto",
    "DisableUserResponseDto",
    "EmailUserResponseDto",
    "EnableUserResponseDto",
    "GetAllUsersResponseDto",
    "GetUserByEmailResponseDto",
    "GetUserByShortUuidResponseDto",
    "GetUserBySubscriptionUuidResponseDto",
    "GetUserByTagResponseDto",
    "GetUserByTelegramIdResponseDto",
    "GetUserByUsernameResponseDto",
    "GetUserByUuidResponseDto",
    "ResetUserTrafficResponseDto",
    "RevokeUserRequestDto",
    "RevokeUserSubscriptionResponseDto",
    "TagsResponseDto",
    "TelegramUserResponseDto",
    "UpdateUserRequestDto",
    "UpdateUserResponseDto",
    "UserActiveInboundsDto",
    "UserLastConnectedNodeDto",
    "UserResponseDto",
    "UsersResponseDto",
    "TagUserResponseDto",
    "GetSubscriptionRequestsResponseDto",
    # Users bulk actions models
    "BulkAllResetTrafficUsersResponseDto",
    "BulkAllUpdateUsersRequestDto",
    "BulkAllUpdateUsersResponseDto",
    "BulkResponseDto",
    "BulkUpdateUsersInternalSquadsRequestDto",
    "UpdateUserFields",
    # Users stats models
    "UserUsageByRange",
    "UserUsageByRangeResponseDto",
    # Config profiles models
    "ConfigProfileDto",
    "CreateConfigProfileRequestDto",
    "CreateConfigProfileResponseDto",
    "DeleteConfigProfileResponseDto",
    "GetAllConfigProfilesResponseDto",
    "GetAllInboundsResponseDto",
    "GetConfigProfileByUuidResponseDto",
    "GetInboundsByProfileUuidResponseDto",
    "InboundDto",
    "UpdateConfigProfileRequestDto",
    "UpdateConfigProfileResponseDto",
    "GetAllConfigProfilesResponsePaginated",
    # Infra billing models
    "CreateInfraBillingNodeRequestDto",
    "CreateInfraBillingNodeResponseDto",
    "CreateInfraProviderRequestDto",
    "CreateInfraProviderResponseDto",
    "DeleteInfraBillingNodeResponseDto",
    "DeleteInfraProviderResponseDto",
    "GetAllInfraBillingHistoryResponseDto",
    "GetAllInfraBillingNodesResponseDto",
    "GetAllInfraProvidersResponseDto",
    "GetInfraBillingHistoryByUuidResponseDto",
    "GetInfraBillingNodeByUuidResponseDto",
    "GetInfraProviderByUuidResponseDto",
    "InfraBillingHistoryDto",
    "InfraBillingNodeDto",
    "InfraProviderDto",
    "NodeDto",
    "UpdateInfraBillingNodeRequestDto",
    "UpdateInfraBillingNodeResponseDto",
    "UpdateInfraProviderRequestDto",
    "UpdateInfraProviderResponseDto",
    # Internal squads models
    "AddUsersToInternalSquadRequestDto",
    "AddUsersToInternalSquadResponseDto",
    "CreateInternalSquadRequestDto",
    "CreateInternalSquadResponseDto",
    "DeleteInternalSquadResponseDto",
    "DeleteUsersFromInternalSquadRequestDto",
    "DeleteUsersFromInternalSquadResponseDto",
    "GetAllInternalSquadsResponseDto",
    "GetInternalSquadByUuidResponseDto",
    "InternalSquadDto",
    "UpdateInternalSquadRequestDto",
    "UpdateInternalSquadResponseDto",
    # Nodes usage history models
    "GetNodeUserUsageByRangeResponseDto",
    "GetNodesUsageByRangeResponseDto",
    "GetUserAccessibleNodesResponseDto",
    "NodeInfoDto",
    "NodeUsageDto",
    "UserUsageDto",
    # Subscription request history models
    "GetAllSubscriptionRequestHistoryResponseDto",
    "GetSubscriptionRequestHistoryStatsResponseDto",
    "SubscriptionRequestHistoryRecord",
    "SubscriptionRequestHistoryData",
    "AppStatItem",
    "HourlyRequestStat", 
    "SubscriptionRequestHistoryStatsData",
    # Webhook models
     # USER
    "LastConnectedNodeDto",
    "InternalSquadDto",
    "BaseUserDto",
    "UserDto",
    "UserEventDto",

    # HWID DEVICES
    "HwidUserDeviceDto",
    "UserHwidDeviceEventDto",

    # SERVICE EVENTS
    "LoginAttemptDto",
    "ServiceEventDto",

    # NODE ENTITIES
    "ConfigProfileInboundDto",
    "InfraProviderDto",
    "NodeDto",
    "NodeEventDto",

    # ERROR EVENTS
    "CustomErrorEventDto",

    # CRM EVENTS
    "CrmEventDto",

    # WEBHOOK PAYLOAD
    "WebhookPayloadDto",
]



from typing import Literal

# ---------------- ENUMS / CONSTANTS ---------------- #

TNodeEvents = Literal[
    "node.created",
    "node.modified",
    "node.disabled",
    "node.enabled",
    "node.deleted",
    "node.connection_lost",
    "node.connection_restored",
    "node.traffic_notify",
]

TUserEvents = Literal[
    "user.created",
    "user.modified",
    "user.deleted",
    "user.revoked",
    "user.disabled",
    "user.enabled",
    "user.limited",
    "user.expired",
    "user.traffic_reset",
    "user.expires_in_72_hours",
    "user.expires_in_48_hours",
    "user.expires_in_24_hours",
    "user.expired_24_hours_ago",
    "user.first_connected",
    "user.bandwidth_usage_threshold_reached",
]

TServiceEvents = Literal[
    "service.panel_started",
    "service.login_attempt_failed",
    "service.login_attempt_success",
]

TErrorsEvents = Literal[
    "errors.bandwidth_usage_threshold_reached_max_notifications",
]

TCRMEvents = Literal[
    "crm.infra_billing_node_payment_in_7_days",
    "crm.infra_billing_node_payment_in_48hrs",
    "crm.infra_billing_node_payment_in_24hrs",
    "crm.infra_billing_node_payment_due_today",
    "crm.infra_billing_node_payment_overdue_24hrs",
    "crm.infra_billing_node_payment_overdue_48hrs",
    "crm.infra_billing_node_payment_overdue_7_days",
]

TUserHwidDevicesEvents = Literal[
    "user_hwid_devices.added",
    "user_hwid_devices.deleted",
]

TResetPeriods = Literal["NO_RESET", "DAY", "WEEK", "MONTH"]
TUsersStatus = Literal["DISABLED", "LIMITED", "EXPIRED", "ACTIVE"]



from typing import Annotated

from httpx import Response
from rapid_api_client import Path
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateApiTokenRequestDto,
    CreateApiTokenResponseDto,
    DeleteApiTokenResponseDto,
    FindAllApiTokensResponseDto,
)
from remnawave.rapid import BaseController, delete, get, post


class APITokensManagementController(BaseController):
    @post("/tokens", response_class=CreateApiTokenResponseDto)
    async def create(
        self,
        body: Annotated[CreateApiTokenRequestDto, PydanticBody()],
    ) -> CreateApiTokenResponseDto:
        """Create new API token"""
        ...

    @delete("/tokens/{uuid}", response_class=DeleteApiTokenResponseDto)
    async def delete(
        self,
        uuid: Annotated[str, Path(description="UUID of the API token")],
    ) -> DeleteApiTokenResponseDto:
        """Delete API token"""
        ...

    @get("/tokens", response_class=FindAllApiTokensResponseDto)
    async def find_all(
        self,
    ) -> FindAllApiTokensResponseDto:
        """Get all API tokens"""
        ...



from datetime import datetime

from pydantic import AliasChoices, BaseModel, Field

from remnawave.enums import ErrorCode


from typing import Any, List, Optional


class ApiErrorResponse(BaseModel):
    timestamp: Optional[datetime] = Field(None, description="Время возникновения ошибки")
    path: Optional[str] = Field(None, description="Путь запроса")
    message: str = Field(..., description="Сообщение об ошибке")
    code: Optional[ErrorCode | str] = Field(
        None,
        validation_alias=AliasChoices("errorCode", "code", "error_code"),
        description="Код ошибки",
    )
    # Support for API v2 error format
    status_code: Optional[int] = Field(None, alias="statusCode")
    errors: Optional[List[Any]] = Field(None, description="Детали ошибок валидации")


class ApiError(Exception):
    def __init__(self, status_code: int, error: ApiErrorResponse):
        self.status_code = status_code
        self.error = error
        super().__init__(
            f"API Error {error.code}: {error.message} (HTTP {status_code})"
        )


class BadRequestError(ApiError):
    """Ошибки клиента (400)"""

    pass


class UnauthorizedError(ApiError):
    """Ошибка авторизации (401)"""

    pass


class ForbiddenError(ApiError):
    """Доступ запрещен (403)"""

    pass


class NotFoundError(ApiError):
    """Ресурс не найден (404)"""

    pass


class ConflictError(ApiError):
    """Конфликт (409)"""

    pass


class ServerError(ApiError):
    """Серверная ошибка (500)"""

    pass



from typing import Annotated

from rapid_api_client import Path

from remnawave.enums import ClientType
from remnawave.models import GetSubscriptionInfoResponseDto
from remnawave.rapid import BaseController, get


class SubscriptionController(BaseController):
    # Public endpoints below
    @get("/sub/{short_uuid}/info", response_class=GetSubscriptionInfoResponseDto)
    async def get_subscription_info_by_short_uuid(
        self,
        short_uuid: Annotated[str, Path(description="Short UUID of the user")],
    ) -> GetSubscriptionInfoResponseDto:
        """None"""
        ...

    @get("/sub/{short_uuid}", response_class=str)
    async def get_subscription(
        self,
        short_uuid: Annotated[str, Path(description="Short UUID of the user")],
    ) -> str:
        """None"""
        ...

    @get("/sub/{short_uuid}/{client_type}", response_class=str)
    async def get_subscription_by_client_type(
        self,
        client_type: Annotated[ClientType, Path(description="Client type")],
        short_uuid: Annotated[str, Path(description="Short UUID of the user")],
    ) -> str:
        """None"""
        ...

    @get("/sub/outline/{short_uuid}/{type}/{encoded_tag}", response_class=str)
    async def get_subscription_with_type(
        self,
        short_uuid: Annotated[str, Path(description="Short UUID of the user")],
        type: Annotated[
            str,
            Path(
                description="Subscription type (required if encodedTag is provided). Only SS is supported for now."
            ),
        ] = "ss",
        encoded_tag: Annotated[
            str,
            Path(
                description="Base64 encoded tag for Outline config. This paramter is optional. It is required only when type=ss."
            ),
        ] = "VGVzdGVy",
    ) -> str:
        """None"""
        ...


from datetime import datetime
from typing import List, Optional, Literal, Union
from uuid import UUID
from pydantic import BaseModel, Field
from pydantic.alias_generators import to_camel
from remnawave.enums import (
    TUsersStatus, TUserEvents, TUserHwidDevicesEvents, TServiceEvents, TNodeEvents, TErrorsEvents, TCRMEvents, TResetPeriods
)
# ---------------- USER ---------------- #

class LastConnectedNodeDto(BaseModel):
    node_name: str
    country_code: str
    connected_at: datetime

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class InternalSquadDto(BaseModel):
    uuid: UUID
    name: str

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class BaseUserDto(BaseModel):
    uuid: UUID
    short_uuid: str
    username: str
    status: TUsersStatus
    used_traffic_bytes: int
    lifetime_used_traffic_bytes: int

    traffic_limit_bytes: int
    traffic_limit_strategy: TResetPeriods
    sub_last_user_agent: Optional[str] = None
    sub_last_opened_at: Optional[datetime] = None

    expire_at: datetime
    sub_revoked_at: Optional[datetime] = None
    last_traffic_reset_at: Optional[datetime] = None

    trojan_password: str
    vless_uuid: UUID
    ss_password: str

    description: Optional[str] = None
    tag: Optional[str] = None
    telegram_id: Optional[int] = None
    email: Optional[str] = None

    hwid_device_limit: Optional[int] = None

    first_connected_at: Optional[datetime] = None
    last_triggered_threshold: int

    online_at: Optional[datetime] = None
    last_connected_node_uuid: Optional[UUID] = None

    created_at: datetime
    updated_at: datetime

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class UserDto(BaseUserDto):
    active_internal_squads: List[InternalSquadDto] = Field(default_factory=list)
    last_connected_node: Optional[LastConnectedNodeDto] = None

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class UserEventDto(BaseModel):
    event_name: TUserEvents
    user: UserDto

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


# ---------------- HWID DEVICES ---------------- #

class HwidUserDeviceDto(BaseModel):
    hwid: str
    user_uuid: UUID
    platform: Optional[str] = None
    os_version: Optional[str] = None
    device_model: Optional[str] = None
    user_agent: Optional[str] = None

    created_at: datetime
    updated_at: datetime

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class UserHwidDeviceEventDto(BaseModel):
    data: dict
    event_name: TUserHwidDevicesEvents

    model_config = {"alias_generator": to_camel, "populate_by_name": True}

    @classmethod
    def build(cls, user: UserDto, hwid_device: HwidUserDeviceDto, event: TUserHwidDevicesEvents):
        return cls(data={"user": user, "hwidUserDevice": hwid_device}, event_name=event)

# ---------------- SERVICE EVENTS ---------------- #

class LoginAttemptDto(BaseModel):
    username: str
    ip: str
    user_agent: str
    description: Optional[str] = None
    password: Optional[str] = None
    
    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class ServiceEventDto(BaseModel):
    event_name: TServiceEvents
    data: dict

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


# ---------------- NODE ENTITIES ---------------- #

class ConfigProfileInboundDto(BaseModel):
    uuid: UUID
    profile_uuid: UUID

    tag: str
    type: str
    network: Optional[str]
    security: Optional[str]
    port: Optional[int]

    raw_inbound: Optional[dict]

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class InfraBillingHistoryDto(BaseModel):
    total_amount: int
    total_bills: int

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class InfraBillingNodeDto(BaseModel):
    node_uuid: UUID
    name: str
    country_code: str

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class InfraProviderDto(BaseModel):
    name: str
    uuid: UUID
    favicon_link: Optional[str] = None
    login_url: Optional[str] = None

    created_at: datetime
    updated_at: datetime

    billing_history: Optional[InfraBillingHistoryDto] = None
    billing_nodes: Optional[List[InfraBillingNodeDto]] = None

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class NodeDto(BaseModel):
    uuid: UUID
    name: str
    address: str
    port: Optional[int] = None
    is_connected: bool
    is_connecting: bool
    is_disabled: bool
    is_node_online: bool
    is_xray_running: bool
    last_status_change: Optional[datetime] = None
    last_status_message: Optional[str] = None

    xray_version: Optional[str] = None
    node_version: Optional[str] = None
    xray_uptime: str

    users_online: Optional[int] = None
    
    is_traffic_tracking_active: bool
    traffic_reset_day: Optional[int] = None
    traffic_limit_bytes: Optional[int] = None
    traffic_used_bytes: Optional[int] = None
    notify_percent: Optional[int] = None

    view_position: int
    country_code: str
    consumption_multiplier: int

    cpu_count: Optional[int] = None
    cpu_model: Optional[str] = None
    total_ram: Optional[str] = None

    created_at: datetime
    updated_at: datetime

    active_config_profile_uuid: Optional[UUID] = None
    active_inbounds: List[ConfigProfileInboundDto] = Field(default_factory=list)

    provider_uuid: Optional[UUID] = None
    provider: Optional[InfraProviderDto] = None

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class NodeEventDto(BaseModel):
    event_name: TNodeEvents
    data: NodeDto

    model_config = {"alias_generator": to_camel, "populate_by_name": True}

# ---------------- ERROR EVENTS ---------------- #

# https://github.com/remnawave/backend/blob/main/src/queue/user-jobs/user-jobs.processor.ts#L224
# Not implemented yet!

class ErrorDto(BaseModel):
    description: str

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class CustomErrorEventDto(BaseModel):
    event_name: TErrorsEvents
    data: ErrorDto

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


# ---------------- CRM EVENTS ---------------- #

class BillingNodeDto(BaseModel):
    provider_name: str
    node_name: str
    next_billing_at: datetime
    login_url: str

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


class CrmEventDto(BaseModel):
    event_name: TCRMEvents
    data: BillingNodeDto

    model_config = {"alias_generator": to_camel, "populate_by_name": True}


# ---------------- WEBHOOK PAYLOAD ---------------- #

class WebhookPayloadDto(BaseModel):
    event: str
    timestamp: datetime
    data: Union[
        UserDto,
        NodeDto,
        HwidUserDeviceDto,
        LoginAttemptDto,
        UserHwidDeviceEventDto,
        BillingNodeDto,
        dict
    ]

    model_config = {"alias_generator": to_camel, "populate_by_name": True}

    @classmethod
    def from_dict(cls, payload: dict) -> "WebhookPayloadDto":
        event = payload.get("event", "")
        data_raw = payload.get("data", {})

        if event.startswith("user."):
            data = UserDto(**data_raw)
        elif event.startswith("user_hwid_devices."):
            user = UserDto(**data_raw["user"])
            hwid_device = HwidUserDeviceDto(**data_raw["hwidUserDevice"])
            data = UserHwidDeviceEventDto.build(
                user=user,
                hwid_device=hwid_device,
                event=event,
            )
        elif event.startswith("node."):
            data = NodeDto(**data_raw)
        elif event.startswith("service."):
            if event.startswith("service.login_attempt"):
                data = LoginAttemptDto(**data_raw)
            else: # service.panel_started - содержит пустой json
                data = data_raw
        elif event.startswith("errors."):
            data = ErrorDto(**data_raw)
        elif event.startswith("crm."):
            data = BillingNodeDto(**data_raw)
        else:
            data = data_raw

        timestamp_raw = payload.get("timestamp")
        if isinstance(timestamp_raw, (int, float)):
            timestamp = datetime.fromtimestamp(timestamp_raw)
        else:
            timestamp = timestamp_raw

        return cls(event=event, data=data, timestamp=timestamp)


from enum import StrEnum


class UserStatus(StrEnum):
    ACTIVE = "ACTIVE"
    DISABLED = "DISABLED"
    LIMITED = "LIMITED"
    EXPIRED = "EXPIRED"


class TrafficLimitStrategy(StrEnum):
    NO_RESET = "NO_RESET"
    DAY = "DAY"
    WEEK = "WEEK"
    MONTH = "MONTH"



from enum import StrEnum


class ErrorCode(StrEnum):
    INTERNAL_SERVER_ERROR = "A001"
    LOGIN_ERROR = "A002"
    UNAUTHORIZED = "A003"
    FORBIDDEN_ROLE_ERROR = "A004"
    CREATE_API_TOKEN_ERROR = "A005"
    DELETE_API_TOKEN_ERROR = "A006"
    REQUESTED_TOKEN_NOT_FOUND = "A007"
    FIND_ALL_API_TOKENS_ERROR = "A008"
    GET_PUBLIC_KEY_ERROR = "A009"
    ENABLE_NODE_ERROR = "A010"
    NODE_NOT_FOUND = "A011"
    CONFIG_NOT_FOUND = "A012"
    UPDATE_CONFIG_ERROR = "A013"
    GET_CONFIG_ERROR = "A014"
    DELETE_MANY_INBOUNDS_ERROR = "A015"
    CREATE_MANY_INBOUNDS_ERROR = "A016"
    FIND_ALL_INBOUNDS_ERROR = "A017"
    CREATE_USER_ERROR = "A018"
    USER_USERNAME_ALREADY_EXISTS = "A019"
    USER_SHORT_UUID_ALREADY_EXISTS = "A020"
    USER_SUBSCRIPTION_UUID_ALREADY_EXISTS = "A021"
    CREATE_USER_WITH_INBOUNDS_ERROR = "A022"
    CANT_GET_CREATED_USER_WITH_INBOUNDS = "A023"
    GET_ALL_USERS_ERROR = "A024"
    USER_NOT_FOUND = "A025"
    GET_USER_BY_ERROR = "A026"
    REVOKE_USER_SUBSCRIPTION_ERROR = "A027"
    DISABLE_USER_ERROR = "A028"
    USER_ALREADY_DISABLED = "A029"
    USER_ALREADY_ENABLED = "A030"
    ENABLE_USER_ERROR = "A031"
    CREATE_NODE_ERROR = "A032"
    NODE_NAME_ALREADY_EXISTS = "A033"
    NODE_ADDRESS_ALREADY_EXISTS = "A034"
    NODE_ERROR_WITH_MSG = "N001"
    NODE_ERROR_500_WITH_MSG = "N002"
    RESTART_NODE_ERROR = "A035"
    GET_CONFIG_WITH_USERS_ERROR = "A036"
    DELETE_USER_ERROR = "A037"
    UPDATE_NODE_ERROR = "A038"
    UPDATE_USER_ERROR = "A039"
    INCREMENT_USED_TRAFFIC_ERROR = "A040"
    GET_ALL_NODES_ERROR = "A041"
    GET_ONE_NODE_ERROR = "A042"
    DELETE_NODE_ERROR = "A043"
    CREATE_HOST_ERROR = "A044"
    HOST_REMARK_ALREADY_EXISTS = "A045"
    HOST_NOT_FOUND = "A046"
    DELETE_HOST_ERROR = "A047"
    GET_USER_STATS_ERROR = "A048"
    UPDATE_USER_WITH_INBOUNDS_ERROR = "A049"
    GET_ALL_HOSTS_ERROR = "A050"
    REORDER_HOSTS_ERROR = "A051"
    UPDATE_HOST_ERROR = "A052"
    CREATE_CONFIG_ERROR = "A053"
    ENABLED_NODES_NOT_FOUND = "A054"
    GET_NODES_USAGE_BY_RANGE_ERROR = "A055"
    RESET_USER_TRAFFIC_ERROR = "A056"
    REORDER_NODES_ERROR = "A057"
    GET_ALL_INBOUNDS_ERROR = "A058"
    BULK_DELETE_USERS_BY_STATUS_ERROR = "A059"
    UPDATE_INBOUND_ERROR = "A060"
    CONFIG_VALIDATION_ERROR = "A061"
    USERS_NOT_FOUND = "A062"
    GET_USER_BY_UNIQUE_FIELDS_NOT_FOUND = "A063"
    UPDATE_EXCEEDED_TRAFFIC_USERS_ERROR = "A064"
    ADMIN_NOT_FOUND = "A065"
    CREATE_ADMIN_ERROR = "A066"
    GET_AUTH_STATUS_ERROR = "A067"
    FORBIDDEN_ONE = "A068"
    FORBIDDEN_TWO = "E000"
    DISABLE_NODE_ERROR = "A069"
    GET_ONE_HOST_ERROR = "A070"
    SUBSCRIPTION_SETTINGS_NOT_FOUND = "A071"
    GET_SUBSCRIPTION_SETTINGS_ERROR = "A072"
    UPDATE_SUBSCRIPTION_SETTINGS_ERROR = "A073"



from typing import Annotated, List
from uuid import UUID

from rapid_api_client.annotations import PydanticBody

from remnawave.enums import UserStatus
from remnawave.models import (
    BulkAllResetTrafficUsersResponseDto,
    BulkAllUpdateUsersRequestDto,
    BulkAllUpdateUsersResponseDto,
    BulkResponseDto,
    BulkUpdateUsersInternalSquadsRequestDto,
    UpdateUserFields,
)
from remnawave.rapid import AttributeBody, BaseController, patch, post


class UsersBulkActionsController(BaseController):
    @post(
        "/users/bulk/delete-by-status",
        response_class=BulkResponseDto,
    )
    async def bulk_delete_users_by_status(
        self, status: Annotated[UserStatus, AttributeBody()]
    ) -> BulkResponseDto:
        """Bulk Delete Users By Status"""
        ...

    @post("/users/bulk/delete", response_class=BulkResponseDto)
    async def bulk_delete_users(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkResponseDto:
        """Bulk Delete Users By UUIDs"""
        ...

    @post(
        "/users/bulk/revoke-subscription",
        response_class=BulkResponseDto,
    )
    async def bulk_revoke_users_subscription(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkResponseDto:
        """Bulk Revoke Users Subscription"""
        ...

    @post("/users/bulk/reset-traffic", response_class=BulkResponseDto)
    async def bulk_reset_user_traffic(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkResponseDto:
        """Bulk Reset User Traffic"""
        ...

    @post("/users/bulk/update", response_class=BulkResponseDto)
    async def bulk_update_users(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
        fields: Annotated[UpdateUserFields, AttributeBody()],
    ) -> BulkResponseDto:
        """Bulk Update Users"""
        ...

    @post("/users/bulk/update-squads", response_class=BulkResponseDto)
    async def bulk_update_users_internal_squads(
        self,
        body: Annotated[BulkUpdateUsersInternalSquadsRequestDto, PydanticBody()],
    ) -> BulkResponseDto:
        """Bulk Update Users Internal Squads"""
        ...

    @post("/users/bulk/all/update", response_class=BulkAllUpdateUsersResponseDto)
    async def bulk_update_all_users(
        self,
        body: Annotated[BulkAllUpdateUsersRequestDto, PydanticBody()],
    ) -> BulkAllUpdateUsersResponseDto:
        """Bulk Update All Users"""
        ...

    @post(
        "/users/bulk/all/reset-traffic",
        response_class=BulkAllResetTrafficUsersResponseDto,
    )
    async def bulk_all_reset_user_traffic(
        self,
    ) -> BulkAllResetTrafficUsersResponseDto:
        """Bulk Reset All Users Traffic"""
        ...



from datetime import datetime
from typing import Annotated, List, Literal, Optional
from uuid import UUID

from pydantic import BaseModel, EmailStr, Field, constr

from remnawave.enums import UserStatus
from remnawave.enums.users import TrafficLimitStrategy


class BulkUpdateUsersInternalSquadsRequestDto(BaseModel):
    uuids: List[UUID]
    active_internal_squads: List[UUID] = Field(serialization_alias="activeInternalSquads")

TagStr = Annotated[
    str,
    Field(min_length=1, max_length=16, pattern=r"^[A-Z0-9_]+$")
]

class UpdateUserFields(BaseModel):
    status: Optional[UserStatus] = None
    traffic_limit_bytes: Optional[int] = Field(
        None, serialization_alias="trafficLimitBytes", strict=True, ge=0
    )
    traffic_limit_strategy: Optional[TrafficLimitStrategy] = Field(
        None, serialization_alias="trafficLimitStrategy"
    )
    expire_at: Optional[datetime] = Field(None, serialization_alias="expireAt")
    description: Optional[str] = None
    tag: Optional[TagStr] = Field(
        None,
        description="Tag for user. Must be uppercase, alphanumeric, and can include underscores. Max length 16 characters."
    )
    telegram_id: Optional[int] = Field(None, serialization_alias="telegramId")
    email: Optional[str] = None


class BulkAllUpdateUsersRequestDto(BaseModel):
    status: Optional[Literal["ACTIVE", "DISABLED", "LIMITED", "EXPIRED"]] = Field(
        "ACTIVE"
    )

    traffic_limit_bytes: Optional[int] = Field(
        None,
        serialization_alias="trafficLimitBytes",
        ge=0,
        description="Traffic limit in bytes. 0 - unlimited"
    )

    traffic_limit_strategy: Optional[Literal["NO_RESET", "DAY", "WEEK", "MONTH"]] = Field(
        None,
        serialization_alias="trafficLimitStrategy",
        description="Traffic limit reset strategy"
    )

    expire_at: Optional[datetime] = Field(
        None,
        serialization_alias="expireAt",
        description="Expiration date"
    )

    description: Optional[str] = None

    telegram_id: Optional[int] = Field(
        None,
        serialization_alias="telegramId"
    )

    email: Optional[str] = None

    tag: Optional[TagStr] = Field(
        None,
        description="Tag for user. Must be uppercase, alphanumeric, and can include underscores. Max length 16 characters."
    )


class BulkResponseDto(BaseModel):
    affected_rows: int = Field(alias="affectedRows")


class BulkAllResetTrafficUsersResponseDto(BaseModel):
    event_sent: bool = Field(alias="eventSent")


class BulkAllUpdateUsersResponseDto(BaseModel):
    event_sent: bool = Field(alias="eventSent")



from datetime import datetime
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class InfraProviderDto(BaseModel):
    uuid: UUID
    name: str
    favicon_link: Optional[str] = Field(None, alias="faviconLink")
    login_url: Optional[str] = Field(None, alias="loginUrl")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class InfraBillingHistoryDto(BaseModel):
    uuid: UUID
    node_uuid: UUID = Field(alias="nodeUuid")
    provider_uuid: UUID = Field(alias="providerUuid")
    provider: InfraProviderDto
    node: "NodeDto"
    next_billing_at: datetime = Field(alias="nextBillingAt")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class NodeDto(BaseModel):
    uuid: UUID
    name: str
    country_code: str = Field(alias="countryCode")


class InfraBillingNodeDto(BaseModel):
    uuid: UUID
    node_uuid: UUID = Field(alias="nodeUuid")
    provider_uuid: UUID = Field(alias="providerUuid")
    provider: InfraProviderDto
    node: NodeDto
    next_billing_at: datetime = Field(alias="nextBillingAt")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


# Provider models
class CreateInfraProviderRequestDto(BaseModel):
    name: str
    favicon_link: Optional[str] = Field(None, serialization_alias="faviconLink")
    login_url: Optional[str] = Field(None, serialization_alias="loginUrl")


class CreateInfraProviderResponseDto(InfraProviderDto):
    pass


class UpdateInfraProviderRequestDto(BaseModel):
    uuid: UUID
    name: Optional[str] = None
    favicon_link: Optional[str] = Field(None, serialization_alias="faviconLink")
    login_url: Optional[str] = Field(None, serialization_alias="loginUrl")


class UpdateInfraProviderResponseDto(InfraProviderDto):
    pass


class AllInfraProvidersData(BaseModel):
    total: int
    providers: List[InfraProviderDto]


class GetAllInfraProvidersResponseDto(AllInfraProvidersData):
    pass


class GetInfraProviderByUuidResponseDto(InfraProviderDto):
    pass


class DeleteInfraProviderResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


# Billing History models
class InfraBillingHistoryData(BaseModel):
    records: List[InfraBillingHistoryDto]
    total: int


class GetAllInfraBillingHistoryResponseDto(InfraBillingHistoryData):
    pass


class GetInfraBillingHistoryByUuidResponseDto(InfraBillingHistoryDto):
    pass


# Billing Nodes models
class CreateInfraBillingNodeRequestDto(BaseModel):
    node_uuid: UUID = Field(serialization_alias="nodeUuid")
    provider_uuid: UUID = Field(serialization_alias="providerUuid")
    next_billing_at: datetime = Field(serialization_alias="nextBillingAt")


class CreateInfraBillingNodeResponseDto(InfraBillingNodeDto):
    pass


class UpdateInfraBillingNodeRequestDto(BaseModel):
    uuid: UUID
    node_uuid: Optional[UUID] = Field(None, serialization_alias="nodeUuid")
    provider_uuid: Optional[UUID] = Field(None, serialization_alias="providerUuid")
    next_billing_at: Optional[datetime] = Field(None, serialization_alias="nextBillingAt")


class UpdateInfraBillingNodeResponseDto(InfraBillingNodeDto):
    pass


class InfraBillingNodesData(BaseModel):
    total_billing_nodes: int = Field(alias="totalBillingNodes")
    total_active_nodes: Optional[int] = Field(None, alias="totalActiveNodes")
    total_spent: Optional[int] = Field(None, alias="totalSpent")
    billing_nodes: List[InfraBillingNodeDto] = Field(alias="billingNodes")


class GetAllInfraBillingNodesResponseDto(InfraBillingNodesData):
    pass


class GetInfraBillingNodeByUuidResponseDto(InfraBillingNodeDto):
    pass


class DeleteInfraBillingNodeResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")



from typing import List, Dict, Any
from uuid import UUID

from pydantic import BaseModel, Field, RootModel


class NodeInfoDto(BaseModel):
    uuid: UUID
    name: str
    country_code: str = Field(alias="countryCode")

class GetUserAccessibleNodesResponse(BaseModel):
    user_uuid: UUID = Field(alias="userUuid")
    nodes: List[NodeInfoDto] = Field(default_factory=list)

class GetUserAccessibleNodesResponseDto(GetUserAccessibleNodesResponse):
    pass


class NodeUsageDto(BaseModel):
    date: str
    upload: int
    download: int


class GetNodesUsageByRangeResponseDto(RootModel[List[NodeUsageDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class UserUsageDto(BaseModel):
    user_uuid: UUID = Field(alias="userUuid")
    username: str
    upload: int
    download: int


class GetNodeUserUsageByRangeResponseDto(RootModel[List[UserUsageDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]



from typing import Annotated, Optional

from pydantic import BaseModel, Field, StringConstraints


class AuthTokenResponseData(BaseModel):
    access_token: str = Field(alias="accessToken")


class RegisterResponseDto(BaseModel):
    access_token: str = Field(alias="accessToken")


class LoginResponseDto(BaseModel):
    access_token: str = Field(alias="accessToken")


class TelegramBotInfo(BaseModel):
    bot_id: int = Field(alias="botId")


class StatusResponseData(BaseModel):
    is_login_allowed: bool = Field(alias="isLoginAllowed")
    is_register_allowed: bool = Field(alias="isRegisterAllowed")
    tg_auth: Optional[TelegramBotInfo] = Field(None, alias="tgAuth")


class GetStatusResponseDto(StatusResponseData):
    pass


class LoginRequestDto(BaseModel):
    username: str
    password: str


class RegisterRequestDto(BaseModel):
    username: str
    password: Annotated[str, StringConstraints(min_length=24)]


class TelegramCallbackRequestDto(BaseModel):
    id: int
    first_name: str
    last_name: Optional[str] = None
    username: Optional[str] = None
    photo_url: Optional[str] = None
    auth_date: int
    hash: str


class TelegramCallbackResponseDto(AuthTokenResponseData):
    pass


# Legacy alias for backward compatibility
StatusResponseDto = GetStatusResponseDto
LoginTelegramRequestDto = TelegramCallbackRequestDto


from functools import partial, wraps
from inspect import signature
from typing import Any, Callable, Coroutine, Type

from httpx import AsyncClient, Response
from pydantic import TypeAdapter
from rapid_api_client.typing import BM, T

from .client import BaseController, CustomRapidParameters


def http(
    method: str,
    path: str,
    response_class: Type[BM | str | bytes | Response] | TypeAdapter[T] = Response,
    timeout: float | None = None,
) -> Callable[
    [Callable], Callable[..., Coroutine[Any, Any, BM | str | bytes | Response | T]]
]:
    def decorator(
        func: Callable,
    ) -> Callable[..., Coroutine[Any, Any, BM | str | bytes | Response | T]]:
        sig = signature(func)
        rapid_parameters = CustomRapidParameters.from_sig(sig)

        @wraps(func)
        async def wrapper(
            api: BaseController, *args, **kwargs
        ) -> BM | str | bytes | Response | T:
            assert isinstance(
                api, BaseController
            ), f"{api} should be an instance of BaseController"
            assert isinstance(
                api.client, AsyncClient
            ), f"{api.client} should be an instance of httpx.AsyncClient"

            # noinspection PyProtectedMember
            request = api._build_request(
                sig, rapid_parameters, method, path, (api,) + args, kwargs, timeout
            )
            response = await api.client.send(request)
            # noinspection PyProtectedMember
            return api._handle_response(response, response_class=response_class)

        return wrapper

    return decorator


get = partial(http, "GET")
post = partial(http, "POST")
delete = partial(http, "DELETE")
put = partial(http, "PUT")
patch = partial(http, "PATCH")



from enum import StrEnum


class ALPN(StrEnum):
    H3 = "h3"
    H2 = "h2"
    HTTP_1_1 = "http/1.1"
    H_COMBINED = "h2,http/1.1"
    H3_H2_H1_COMBINED = "h3,h2,http/1.1"
    H3_H2_COMBINED = "h3,h2"



import datetime
from typing import List
from uuid import UUID

from pydantic import BaseModel, Field


class UserUsageByRange(BaseModel):
    user_uuid: UUID = Field(alias="userUuid")
    node_uuid: UUID = Field(alias="nodeUuid")
    node_name: str = Field(alias="nodeName")
    total: int
    date: datetime.date


class UserUsageByRangeResponseDto(List[UserUsageByRange]):
    pass



from datetime import datetime
from typing import Annotated, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, StringConstraints, RootModel

from remnawave.models.internal_squads import InboundsDto


class ExcludedInbounds(BaseModel):
    uuid: UUID
    tag: str
    type: str
    network: Optional[str] = None
    security: Optional[str] = None


class RestartEventResponse(BaseModel):
    event_sent: bool = Field(alias="eventSent")


class DeleteResponse(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class ReorderNodeItem(BaseModel):
    view_position: int = Field(serialization_alias="viewPosition")
    uuid: UUID


class NodeConfigProfileDto(BaseModel):
    active_config_profile_uuid: UUID = Field(alias="activeConfigProfileUuid")
    active_inbounds: List[InboundsDto] = Field(alias="activeInbounds")


class NodeConfigProfileRequestDto(BaseModel):
    active_config_profile_uuid: UUID = Field(alias="activeConfigProfileUuid")
    active_inbounds: List[UUID] = Field(alias="activeInbounds")


class CreateNodeRequestDto(BaseModel):
    name: Annotated[str, StringConstraints(min_length=3)]
    address: Annotated[str, StringConstraints(min_length=2)]
    port: Optional[int] = Field(None, strict=True, ge=1)
    is_traffic_tracking_active: Optional[bool] = Field(
        None,
        serialization_alias="isTrafficTrackingActive",
    )
    traffic_limit_bytes: Optional[int] = Field(
        None, serialization_alias="trafficLimitBytes", strict=True, ge=0
    )
    notify_percent: Optional[int] = Field(
        None, serialization_alias="notifyPercent", strict=True, ge=0
    )
    traffic_reset_day: Optional[int] = Field(
        None, serialization_alias="trafficResetDay", strict=True, ge=1
    )
    excluded_inbounds: Optional[List[UUID]] = Field(
        None, serialization_alias="excludedInbounds"
    )
    country_code: Annotated[Optional[str], StringConstraints(max_length=2)] = Field(
        None, serialization_alias="countryCode"
    )
    consumption_multiplier: Optional[float] = Field(
        None, serialization_alias="consumptionMultiplier"
    )
    config_profile: NodeConfigProfileRequestDto = Field(
        serialization_alias="configProfile"
    )
    provider_uuid: Optional[UUID] = Field(None, serialization_alias="providerUuid")


class UpdateNodeRequestDto(BaseModel):
    uuid: UUID
    name: Annotated[Optional[str], StringConstraints(min_length=3)] = None
    address: Annotated[Optional[str], StringConstraints(min_length=2)] = None
    port: Optional[int] = None
    is_traffic_tracking_active: Optional[bool] = Field(
        None, serialization_alias="isTrafficTrackingActive"
    )
    traffic_limit_bytes: Optional[float] = Field(
        None, serialization_alias="trafficLimitBytes"
    )
    notify_percent: Optional[float] = Field(None, serialization_alias="notifyPercent")
    traffic_reset_day: Optional[float] = Field(
        None, serialization_alias="trafficResetDay"
    )
    excluded_inbounds: Optional[List[UUID]] = Field(
        None, serialization_alias="excludedInbounds"
    )
    country_code: Annotated[Optional[str], StringConstraints(max_length=2)] = Field(
        None, serialization_alias="countryCode"
    )
    consumption_multiplier: Optional[float] = Field(
        None, serialization_alias="consumptionMultiplier"
    )


class ReorderNodeRequestDto(BaseModel):
    nodes: List[ReorderNodeItem]


class NodeResponseDto(BaseModel):
    uuid: UUID
    name: str
    address: str
    port: Optional[int] = None
    is_connected: bool = Field(alias="isConnected")
    is_disabled: bool = Field(alias="isDisabled")
    is_connecting: bool = Field(alias="isConnecting")
    is_node_online: bool = Field(alias="isNodeOnline")
    is_xray_running: bool = Field(alias="isXrayRunning")
    last_status_change: Optional[datetime] = Field(None, alias="lastStatusChange")
    last_status_message: Optional[str] = Field(None, alias="lastStatusMessage")
    xray_version: Optional[str] = Field(None, alias="xrayVersion")
    xray_uptime: str = Field(alias="xrayUptime")
    is_traffic_tracking_active: bool = Field(alias="isTrafficTrackingActive")
    traffic_reset_day: Optional[int] = Field(None, alias="trafficResetDay")
    traffic_limit_bytes: Optional[float] = Field(None, alias="trafficLimitBytes")
    traffic_used_bytes: Optional[float] = Field(None, alias="trafficUsedBytes")
    notify_percent: Optional[int] = Field(None, alias="notifyPercent")
    users_online: Optional[int] = Field(None, alias="usersOnline")
    view_position: int = Field(alias="viewPosition")
    country_code: str = Field(alias="countryCode")
    consumption_multiplier: float = Field(alias="consumptionMultiplier")
    cpu_count: Optional[int] = Field(None, alias="cpuCount")
    cpu_model: Optional[str] = Field(None, alias="cpuModel")
    total_ram: Optional[str] = Field(None, alias="totalRam")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")
    config_profile: NodeConfigProfileDto = Field(alias="configProfile")


class NodesResponseDto(RootModel[List[NodeResponseDto]]):
    root: List[NodeResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateNodeResponseDto(NodeResponseDto):
    pass


class UpdateNodeResponseDto(NodeResponseDto):
    pass


class GetOneNodeResponseDto(NodeResponseDto):
    pass


class GetAllNodesResponseDto(RootModel[List[NodeResponseDto]]):
    root: List[NodeResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class EnableNodeResponseDto(NodeResponseDto):
    pass


class DisableNodeResponseDto(NodeResponseDto):
    pass


class RestartNodeResponseDto(BaseModel):
    message: str


class RestartAllNodesResponseDto(BaseModel):
    message: str


class ReorderNodeResponseDto(RootModel[List[NodeResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class DeleteNodeResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")

    def __bool__(self):
        return self.is_deleted


class RestartAllNodesRequestDto(BaseModel):
    force_restart: bool = Field(default=False, alias="forceRestart")



from typing import Annotated

from rapid_api_client import Path, Query

from remnawave.models import GetUserUsageByRangeResponseDto
from remnawave.rapid import BaseController, get


class UsersStatsController(BaseController):
    @get(
        "/users/stats/usage/{uuid}/range",
        response_class=GetUserUsageByRangeResponseDto,
    )
    async def get_user_usage_by_range(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
        start: Annotated[str, Query(description="Start date in ISO format")],
        end: Annotated[str, Query(description="End date in ISO format")],
    ) -> GetUserUsageByRangeResponseDto:
        """Get User Usage By Range"""
        ...



from datetime import datetime
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class SubscriptionRequestHistoryRecord(BaseModel):
    id: int
    user_uuid: UUID = Field(alias="userUuid")
    request_ip: Optional[str] = Field(alias="requestIp")
    user_agent: Optional[str] = Field(alias="userAgent")
    request_at: datetime = Field(alias="requestAt")


class SubscriptionRequestHistoryData(BaseModel):
    records: List[SubscriptionRequestHistoryRecord]
    total: float


class GetAllSubscriptionRequestHistoryResponseDto(SubscriptionRequestHistoryData):
    pass


class AppStatItem(BaseModel):
    app: str
    count: float


class HourlyRequestStat(BaseModel):
    date_time: datetime = Field(alias="dateTime")
    request_count: float = Field(alias="requestCount")


class SubscriptionRequestHistoryStatsData(BaseModel):
    by_parsed_app: List[AppStatItem] = Field(alias="byParsedApp")
    hourly_request_stats: List[HourlyRequestStat] = Field(alias="hourlyRequestStats")


class GetSubscriptionRequestHistoryStatsResponseDto(SubscriptionRequestHistoryStatsData):
    pass


from typing import Annotated

from rapid_api_client import Path, Query
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateInfraBillingNodeRequestDto,
    CreateInfraBillingNodeResponseDto,
    CreateInfraProviderRequestDto,
    CreateInfraProviderResponseDto,
    DeleteInfraBillingNodeResponseDto,
    DeleteInfraProviderResponseDto,
    GetAllInfraBillingHistoryResponseDto,
    GetAllInfraBillingNodesResponseDto,
    GetAllInfraProvidersResponseDto,
    GetInfraBillingHistoryByUuidResponseDto,
    GetInfraBillingNodeByUuidResponseDto,
    GetInfraProviderByUuidResponseDto,
    UpdateInfraBillingNodeRequestDto,
    UpdateInfraBillingNodeResponseDto,
    UpdateInfraProviderRequestDto,
    UpdateInfraProviderResponseDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class InfraBillingController(BaseController):
    @get("/infra-billing/providers", response_class=GetAllInfraProvidersResponseDto)
    async def get_all_infra_providers(self) -> GetAllInfraProvidersResponseDto:
        """Get all infra providers"""
        ...

    @post("/infra-billing/providers", response_class=CreateInfraProviderResponseDto)
    async def create_infra_provider(
        self,
        body: Annotated[CreateInfraProviderRequestDto, PydanticBody()],
    ) -> CreateInfraProviderResponseDto:
        """Create infra provider"""
        ...

    @patch("/infra-billing/providers", response_class=UpdateInfraProviderResponseDto)
    async def update_infra_provider(
        self,
        body: Annotated[UpdateInfraProviderRequestDto, PydanticBody()],
    ) -> UpdateInfraProviderResponseDto:
        """Update infra provider"""
        ...

    @get("/infra-billing/providers/{uuid}", response_class=GetInfraProviderByUuidResponseDto)
    async def get_infra_provider_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the infra provider")],
    ) -> GetInfraProviderByUuidResponseDto:
        """Get infra provider by uuid"""
        ...

    @delete("/infra-billing/providers/{uuid}", response_class=DeleteInfraProviderResponseDto)
    async def delete_infra_provider_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the infra provider")],
    ) -> DeleteInfraProviderResponseDto:
        """Delete infra provider"""
        ...

    @get("/infra-billing/history", response_class=GetAllInfraBillingHistoryResponseDto)
    async def get_all_infra_billing_history(
        self,
        start: Annotated[int, Query(default=0, ge=0, description="Index to start pagination from")],
        size: Annotated[int, Query(default=25, ge=1, description="Number of entries per page")],
    ) -> GetAllInfraBillingHistoryResponseDto:
        """Get all infra billing history"""
        ...

    @get("/infra-billing/history/{uuid}", response_class=GetInfraBillingHistoryByUuidResponseDto)
    async def get_infra_billing_history_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the billing history entry")],
    ) -> GetInfraBillingHistoryByUuidResponseDto:
        """Get infra billing history by uuid"""
        ...

    @get("/infra-billing/nodes", response_class=GetAllInfraBillingNodesResponseDto)
    async def get_all_infra_billing_nodes(
        self,
        start: Annotated[int, Query(default=0, ge=0, description="Index to start pagination from")],
        size: Annotated[int, Query(default=25, ge=1, description="Number of entries per page")],
    ) -> GetAllInfraBillingNodesResponseDto:
        """Get all infra billing nodes"""
        ...

    @patch("/infra-billing/nodes", response_class=UpdateInfraBillingNodeResponseDto)
    async def update_infra_billing_node(
        self,
        body: Annotated[UpdateInfraBillingNodeRequestDto, PydanticBody()],
    ) -> UpdateInfraBillingNodeResponseDto:
        """Update infra billing node"""
        ...

    @post("/infra-billing/nodes", response_class=CreateInfraBillingNodeResponseDto)
    async def create_infra_billing_node(
        self,
        body: Annotated[CreateInfraBillingNodeRequestDto, PydanticBody()],
    ) -> CreateInfraBillingNodeResponseDto:
        """Create infra billing node"""
        ...

    @get("/infra-billing/nodes/{uuid}", response_class=GetInfraBillingNodeByUuidResponseDto)
    async def get_infra_billing_node_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the infra billing node")],
    ) -> GetInfraBillingNodeByUuidResponseDto:
        """Get infra billing node by uuid"""
        ...

    @delete("/infra-billing/nodes/{uuid}", response_class=DeleteInfraBillingNodeResponseDto)
    async def delete_infra_billing_node_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the infra billing node")],
    ) -> DeleteInfraBillingNodeResponseDto:
        """Delete infra billing node"""
        ...



from .handler import handle_api_error
from .general import (
    ConflictError,
    ApiErrorResponse,
    BadRequestError,
    NotFoundError,
    ForbiddenError,
    UnauthorizedError,
    ServerError,
    ApiError,
)

__all__ = [
    "handle_api_error",
    "ApiError",
    "ApiErrorResponse",
    "NotFoundError",
    "BadRequestError",
    "ForbiddenError",
    "UnauthorizedError",
    "ConflictError",
    "ServerError",
]



from pydantic import BaseModel, Field


class AddInboundToUsersResponseDto(BaseModel):
    is_success: bool = Field(alias="isSuccess")


class RemoveInboundFromNodesResponseDto(BaseModel):
    is_success: bool = Field(alias="isSuccess")


class RemoveInboundFromUsersResponseDto(BaseModel):
    is_success: bool = Field(alias="isSuccess")


class AddInboundToNodesResponseDto(BaseModel):
    is_success: bool = Field(alias="isSuccess")



from datetime import datetime
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class InboundsDto(BaseModel):
    uuid: UUID
    profile_uuid: UUID = Field(alias="profileUuid")
    tag: str
    type: str
    network: Optional[str] = Field(default=None)
    security: Optional[str] = Field(default=None)
    port: Optional[float] = Field(default=None)
    raw_inbound: Optional[dict] = Field(default=None, alias="rawInbound")


class InfoDto(BaseModel):
    members_count: int = Field(alias="membersCount")
    inbounds_count: int = Field(alias="inboundsCount")


class InternalSquadDto(BaseModel):
    uuid: UUID
    name: str
    info: Optional[InfoDto] = Field(default=None)
    inbounds: List[InboundsDto] = Field(default_factory=list)
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class CreateInternalSquadRequestDto(BaseModel):
    name: str
    inbounds: List[UUID] = Field(default_factory=list)


class CreateInternalSquadResponseDto(InternalSquadDto):
    pass


class UpdateInternalSquadRequestDto(BaseModel):
    uuid: UUID
    inbounds: List[UUID] = Field(default_factory=list)
    name: Optional[str] = Field(None, pattern=r"^[A-Za-z0-9_-]+$")


class UpdateInternalSquadResponseDto(InternalSquadDto):
    pass


class GetAllInternalSquadsResponse(BaseModel):
    total: int
    internal_squads: List[InternalSquadDto] = Field(alias="internalSquads")


class GetAllInternalSquadsResponseDto(GetAllInternalSquadsResponse):
    pass


class GetInternalSquadByUuidResponseDto(InternalSquadDto):
    pass


class DeleteInternalSquadResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class AddUsersToInternalSquadRequestDto(BaseModel):
    user_uuids: List[UUID] = Field(alias="userUuids")


class BulkActionsResponseDto(BaseModel):
    event_sent: bool = Field(alias="eventSent")


class AddUsersToInternalSquadResponseDto(BulkActionsResponseDto):
    pass


class DeleteUsersFromInternalSquadRequestDto(BaseModel):
    user_uuids: List[UUID] = Field(alias="userUuids")


class DeleteUsersFromInternalSquadResponseDto(BulkActionsResponseDto):
    pass


class AccessibleNodeDto(BaseModel):
    uuid: UUID
    name: str = Field(alias="nodeName")
    country_code: Optional[str] = Field(default=None, alias="countryCode")
    config_profile_uuid: Optional[UUID] = Field(default=None, alias="configProfileUuid")
    config_profile_name: Optional[str] = Field(default=None, alias="configProfileName")
    active_inbounds: List[Optional[UUID]] = Field(
        default_factory=list, alias="activeInbounds"
    )


class GetInternalSquadAccessibleNodesResponseDto(BaseModel):
    squad_uuid: UUID = Field(alias="squadUuid")
    accessible_nodes: List[AccessibleNodeDto] = Field(alias="accessibleNodes")



from typing import Annotated

from rapid_api_client import Path
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    AddUsersToInternalSquadRequestDto,
    AddUsersToInternalSquadResponseDto,
    CreateInternalSquadRequestDto,
    CreateInternalSquadResponseDto,
    DeleteInternalSquadResponseDto,
    DeleteUsersFromInternalSquadRequestDto,
    DeleteUsersFromInternalSquadResponseDto,
    GetAllInternalSquadsResponseDto,
    GetInternalSquadByUuidResponseDto,
    UpdateInternalSquadRequestDto,
    UpdateInternalSquadResponseDto,
    GetInternalSquadAccessibleNodesResponseDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class InternalSquadsController(BaseController):
    @get("/internal-squads", response_class=GetAllInternalSquadsResponseDto)
    async def get_internal_squads(self) -> GetAllInternalSquadsResponseDto:
        """Get all internal squads"""
        ...

    @post("/internal-squads", response_class=CreateInternalSquadResponseDto)
    async def create_internal_squad(
        self,
        body: Annotated[CreateInternalSquadRequestDto, PydanticBody()],
    ) -> CreateInternalSquadResponseDto:
        """Create internal squad"""
        ...

    @patch("/internal-squads", response_class=UpdateInternalSquadResponseDto)
    async def update_internal_squad(
        self,
        body: Annotated[UpdateInternalSquadRequestDto, PydanticBody()],
    ) -> UpdateInternalSquadResponseDto:
        """Update internal squad"""
        ...

    @get("/internal-squads/{uuid}", response_class=GetInternalSquadByUuidResponseDto)
    async def get_internal_squad_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the internal squad")],
    ) -> GetInternalSquadByUuidResponseDto:
        """Get internal squad by uuid"""
        ...

    @delete("/internal-squads/{uuid}", response_class=DeleteInternalSquadResponseDto)
    async def delete_internal_squad(
        self,
        uuid: Annotated[str, Path(description="UUID of the internal squad")],
    ) -> DeleteInternalSquadResponseDto:
        """Delete internal squad"""
        ...

    @post(
        "/internal-squads/{uuid}/bulk-actions/add-users",
        response_class=AddUsersToInternalSquadResponseDto,
    )
    async def add_users_to_internal_squad(
        self,
        uuid: Annotated[str, Path(description="UUID of the internal squad")],
    ) -> AddUsersToInternalSquadResponseDto:
        """Add users to internal squad"""
        ...

    @delete(
        "/internal-squads/{uuid}/bulk-actions/remove-users",
        response_class=DeleteUsersFromInternalSquadResponseDto,
    )
    async def remove_users_from_internal_squad(
        self,
        uuid: Annotated[str, Path(description="UUID of the internal squad")],
    ) -> DeleteUsersFromInternalSquadResponseDto:
        """Delete users from internal squad"""
        ...

    @get(
        "/internal-squads/{uuid}/accessible-nodes",
        response_class=GetInternalSquadAccessibleNodesResponseDto,
    )
    async def get_accessible_nodes(
        self,
        uuid: Annotated[str, Path(description="UUID of the internal squad")],
    ) -> GetInternalSquadAccessibleNodesResponseDto:
        """Get accessible nodes for internal squad"""
        ...



from enum import StrEnum


class ClientType(StrEnum):
    STASH = "stash"
    SINGBOX = "singbox"
    SINGBOX_LEGACY = "singbox_legacy"
    MIHOMO = "mihomo"
    JSON = "json"
    CLASH = "clash"
    V2RAY_JSON = "v2ray-json"



from pydantic import BaseModel, Field


class PubKeyData(BaseModel):
    pub_key: str = Field(alias="pubKey")


class GetPubKeyResponseDto(BaseModel):
    pub_key: str = Field(alias="pubKey")


# Legacy alias for backward compatibility
PubKeyResponseDto = PubKeyData



import datetime
from typing import List

from pydantic import BaseModel, Field


class NodeStatistic(BaseModel):
    node_name: str = Field(alias="nodeName")
    date: datetime.date
    total_bytes: int = Field(alias="totalBytes")


class NodesStatisticResponseDto(BaseModel):
    last_seven_days: List[NodeStatistic] = Field(alias="lastSevenDays")


class BandwidthStatistic(BaseModel):
    current: str
    previous: str
    difference: str


class BandwidthStatisticResponseDto(BaseModel):
    last_two_days: BandwidthStatistic = Field(alias="bandwidthLastTwoDays")
    last_seven_days: BandwidthStatistic = Field(alias="bandwidthLastSevenDays")
    last_30_days: BandwidthStatistic = Field(alias="bandwidthLast30Days")
    calendar_month: BandwidthStatistic = Field(alias="bandwidthCalendarMonth")
    current_year: BandwidthStatistic = Field(alias="bandwidthCurrentYear")


class CPUStatistic(BaseModel):
    cores: int
    physical_cores: int = Field(alias="physicalCores")


class MemoryStatistic(BaseModel):
    total: int
    free: int
    used: int
    active: int
    available: int


class StatusCounts(BaseModel):
    active: int = Field(alias="ACTIVE")
    disabled: int = Field(alias="DISABLED")
    limited: int = Field(alias="LIMITED")
    expired: int = Field(alias="EXPIRED")


class UsersStatistic(BaseModel):
    status_counts: StatusCounts = Field(alias="statusCounts")
    total_users: int = Field(alias="totalUsers")
    total_traffic_bytes: int = Field(alias="totalTrafficBytes")


class OnlineStatistic(BaseModel):
    last_day: int = Field(alias="lastDay")
    last_week: int = Field(alias="lastWeek")
    never_online: int = Field(alias="neverOnline")
    online_now: int = Field(alias="onlineNow")


class NodesStatistic(BaseModel):
    total_online: int = Field(alias="totalOnline")


class StatisticResponseDto(BaseModel):
    cpu: CPUStatistic
    memory: MemoryStatistic
    uptime: float
    timestamp: int
    users: UsersStatistic
    online_stats: OnlineStatistic = Field(alias="onlineStats")
    nodes: NodesStatistic


class PM2Stat(BaseModel):
    name: str
    memory: str
    cpu: str


class RemnawaveHealthData(BaseModel):
    pm2_stats: List[PM2Stat] = Field(alias="pm2Stats")


class GetStatsResponseDto(StatisticResponseDto):
    pass


class GetBandwidthStatsResponseDto(BaseModel):
    last_two_days: BandwidthStatistic = Field(alias="bandwidthLastTwoDays")
    last_seven_days: BandwidthStatistic = Field(alias="bandwidthLastSevenDays")
    last_30_days: BandwidthStatistic = Field(alias="bandwidthLast30Days")
    calendar_month: BandwidthStatistic = Field(alias="bandwidthCalendarMonth")
    current_year: BandwidthStatistic = Field(alias="bandwidthCurrentYear")


class GetNodesStatisticsResponseDto(BaseModel):
    last_seven_days: List[NodeStatistic] = Field(alias="lastSevenDays")


class GetRemnawaveHealthResponseDto(BaseModel):
    pm2_stats: List[PM2Stat] = Field(alias="pm2Stats")


class NodeMetric(BaseModel):
    uuid: str
    name: str
    address: str
    is_online: bool = Field(alias="isOnline")
    cpu_usage: float = Field(alias="cpuUsage")
    memory_usage: float = Field(alias="memoryUsage")
    network_upload: int = Field(alias="networkUpload")
    network_download: int = Field(alias="networkDownload")
    uptime: int
    last_seen: datetime.datetime = Field(alias="lastSeen")
    connected_users: int = Field(alias="connectedUsers")


class GetNodesMetricsResponseDto(BaseModel):
    nodes: List[NodeMetric]

class X25519KeyPair(BaseModel):
    public_key: str = Field(alias="publicKey")
    private_key: str = Field(alias="privateKey")

class GetX25519KeyPairResponseDto(BaseModel):
    key_pairs: List[X25519KeyPair] = Field(alias="keyPairs")


from typing import Annotated
from uuid import UUID

from remnawave.models import (
    CreateUserHwidDeviceResponseDto,
    DeleteUserHwidDeviceResponseDto,
    GetUserHwidDevicesResponseDto,
    GetHwidStatisticsResponseDto,
    CreateHWIDUser,
    HWIDDeleteRequest,
    DeleteUserAllHwidDeviceRequestDto
)
from rapid_api_client import Path, PydanticBody
from remnawave.rapid import AttributeBody, BaseController, post, get


class HWIDUserController(BaseController):
    @get("/hwid/devices", response_class=GetUserHwidDevicesResponseDto)
    async def get_hwid_users(
        self,
        size: Annotated[int | None, AttributeBody()] = None,
        start: Annotated[int | None, AttributeBody()] = None,
    ) -> GetUserHwidDevicesResponseDto:
        """Get all user HWID devices"""
        ...

    @get("/hwid/devices/stats", response_class=GetHwidStatisticsResponseDto)
    async def get_hwid_stats(
        self,
    ) -> GetHwidStatisticsResponseDto:
        """Get HWID statistics"""
        ...

    @post("/hwid/devices", response_class=CreateUserHwidDeviceResponseDto)
    async def add_hwid_to_users(
        self,
        body: Annotated[CreateHWIDUser, PydanticBody()],
    ) -> CreateUserHwidDeviceResponseDto:
        """Create a user HWID device"""
        ...

    @post("/hwid/devices/delete", response_class=DeleteUserHwidDeviceResponseDto)
    async def delete_hwid_to_user(
        self,
        body: Annotated[HWIDDeleteRequest, PydanticBody()],
    ) -> DeleteUserHwidDeviceResponseDto:
        """Delete a user HWID device"""
        ...

    @post("/hwid/devices/delete-all", response_class=DeleteUserHwidDeviceResponseDto)
    async def delete_all_hwid_user(
        self,
        body: Annotated[DeleteUserAllHwidDeviceRequestDto, PydanticBody()],
    ) -> DeleteUserHwidDeviceResponseDto:
        """Delete all user HWID devices"""
        ...

    @get("/hwid/devices/{uuid}", response_class=GetUserHwidDevicesResponseDto)
    async def get_hwid_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the User")],
    ) -> GetUserHwidDevicesResponseDto:
        """Get a user HWID device"""
        ...



from typing import Annotated, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, StringConstraints, RootModel

from remnawave.enums import ALPN, Fingerprint, SecurityLayer


class ReorderResponse(BaseModel):
    is_updated: bool = Field(alias="isUpdated")


class DeleteResponse(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class ReorderHostItem(BaseModel):
    view_position: int = Field(serialization_alias="viewPosition")
    uuid: UUID


class ReorderHostRequestDto(BaseModel):
    hosts: List[ReorderHostItem]


class UpdateHostRequestDto(BaseModel):
    uuid: UUID
    inbound_uuid: Optional[UUID] = Field(None, serialization_alias="inboundUuid")
    remark: Annotated[Optional[str], StringConstraints(max_length=40)] = None
    address: Optional[str] = None
    port: Optional[int] = None
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[ALPN] = None
    fingerprint: Optional[Fingerprint] = None
    allow_insecure: Optional[bool] = Field(None, serialization_alias="allowInsecure")
    is_disabled: Optional[bool] = Field(None, serialization_alias="isDisabled")
    security_layer: Optional[SecurityLayer] = Field(
        None, serialization_alias="securityLayer"
    )
    server_description: Optional[str] = Field(
        None, serialization_alias="serverDescription", max_length=30
    )
    mux_params: Optional[str] = Field(
        None,
        serialization_alias="muxParams",
    )
    sockopt_params: Optional[str] = Field(
        None,
        serialization_alias="sockoptParams",
    )
    tag: Optional[Annotated[str, StringConstraints(max_length=32)]] = Field(
        None, serialization_alias="tag"
    )
    is_hidden: Optional[bool] = Field(
        None,
        serialization_alias="isHidden",
    )
    override_sni_from_address: Optional[bool] = Field(
        None,
        serialization_alias="overrideSniFromAddress",
    )
    vless_route_id: Optional[int] = Field(
        None,
        serialization_alias="vlessRouteId",
        ge=0,
        le=65535
    )
    shuffle_host: Optional[bool] = Field(
        None,
        serialization_alias="shuffleHost",
    )
    mihomo_x25519: Optional[bool] = Field(
        None,
        serialization_alias="mihomoX25519",
    )
    x_http_extra_params: Optional[str] = Field(
        None,
        serialization_alias="xHttpExtraParams",
    )
    nodes: Optional[List[str]] = None


class HostInboundData(BaseModel):
    config_profile_uuid: Optional[UUID] = Field(alias="configProfileUuid")
    config_profile_inbound_uuid: Optional[UUID] = Field(alias="configProfileInboundUuid")


class HostResponseDto(BaseModel):
    uuid: UUID
    view_position: int = Field(alias="viewPosition")
    remark: str
    address: str
    port: int
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[ALPN] = None
    fingerprint: Optional[Fingerprint] = None
    x_http_extra_params: Optional[str] = Field(
        None,
        alias="xHttpExtraParams",
    )
    mux_params: Optional[str] = Field(
        None,
        alias="muxParams",
    )
    sockopt_params: Optional[str] = Field(
        None,
        alias="sockoptParams",
    )
    inbound: HostInboundData
    server_description: Optional[str] = Field(
        None, alias="serverDescription"
    )
    tag: Optional[str] = None
    vless_route_id: Optional[int] = Field(
        None,
        alias="vlessRouteId",
    )
    shuffle_host: bool = Field(alias="shuffleHost")
    mihomo_x25519: bool = Field(alias="mihomoX25519")
    nodes: List[str]
    is_disabled: bool = Field(
        default=False,
        alias="isDisabled",
    )
    security_layer: SecurityLayer = Field(
        default=SecurityLayer.DEFAULT,
        alias="securityLayer",
    )
    is_hidden: bool = Field(
        default=False,
        alias="isHidden",
    )
    override_sni_from_address: bool = Field(
        default=False,
        alias="overrideSniFromAddress",
    )
    allow_insecure: bool = Field(
        default=False,
        alias="allowInsecure",
    )

    # Legacy compatibility property
    @property
    def inbound_uuid(self) -> Optional[UUID]:
        return self.inbound.config_profile_inbound_uuid


class HostsResponseDto(List[HostResponseDto]):
    pass


class CreateHostResponseDto(HostResponseDto):
    pass


class UpdateHostResponseDto(HostResponseDto):
    pass


class GetAllHostTagsResponseDto(BaseModel):
    tags: list[str] = None


class GetAllHostsResponseDto(RootModel[List[HostResponseDto]]):
    root: List[HostResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class GetOneHostResponseDto(HostResponseDto):
    pass


class ReorderHostResponseDto(BaseModel):
    is_updated: bool = Field(alias="isUpdated", default=True)


class DeleteHostResponseDto(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class CreateHostInboundData(BaseModel):
    config_profile_uuid: Optional[UUID] = Field(serialization_alias="configProfileUuid")
    config_profile_inbound_uuid: Optional[UUID] = Field(
        serialization_alias="configProfileInboundUuid"
    )


class CreateHostRequestDto(BaseModel):
    inbound: CreateHostInboundData
    remark: Annotated[str, StringConstraints(max_length=40)]
    address: str
    port: int
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[ALPN] = None
    fingerprint: Optional[Fingerprint] = None
    x_http_extra_params: Optional[str] = Field(
        None,
        serialization_alias="xHttpExtraParams",
    )
    mux_params: Optional[str] = Field(
        None,
        serialization_alias="muxParams",
    )
    sockopt_params: Optional[str] = Field(
        None,
        serialization_alias="sockoptParams",
    )
    server_description: Optional[str] = Field(
        None, serialization_alias="serverDescription", max_length=30
    )
    tag: Optional[Annotated[str, StringConstraints(max_length=32)]] = Field(
        None, serialization_alias="tag"
    )
    vless_route_id: Optional[int] = Field(
        None,
        serialization_alias="vlessRouteId",
        ge=0,
        le=65535
    )
    shuffle_host: bool = Field(
        False,
        serialization_alias="shuffleHost",
    )
    mihomo_x25519: bool = Field(
        False,
        serialization_alias="mihomoX25519",
    )
    nodes: List[str] = Field(default_factory=list)
    allow_insecure: bool = Field(
        False,
        serialization_alias="allowInsecure",
    )
    is_disabled: bool = Field(
        False, 
        serialization_alias="isDisabled",
    )
    security_layer: SecurityLayer = Field(
        SecurityLayer.DEFAULT, 
        serialization_alias="securityLayer",
    )
    is_hidden: bool = Field(
        False,
        serialization_alias="isHidden",
    )
    override_sni_from_address: bool = Field(
        False, 
        serialization_alias="overrideSniFromAddress",
    )

    # Legacy compatibility property
    @property
    def inbound_uuid(self) -> Optional[UUID]:
        return self.inbound.config_profile_inbound_uuid

    # Constructor compatibility - support old-style inbound_uuid
    def __init__(
        self,
        inbound_uuid: Optional[UUID] = None,
        config_profile_uuid: Optional[UUID] = None,
        **data,
    ):
        if inbound_uuid is not None and "inbound" not in data:
            # Legacy mode: create inbound object from UUID
            # Use hardcoded config_profile_uuid from API response for compatibility
            data["inbound"] = CreateHostInboundData(
                config_profile_uuid=config_profile_uuid
                or UUID("107541f1-ae1a-4e2d-9dec-7297557b5125"),
                config_profile_inbound_uuid=inbound_uuid,
            )
        super().__init__(**data)


from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, ConfigDict, Field


class CreateApiTokenRequestDto(BaseModel):
    token_name: str = Field(serialization_alias="tokenName")


class CreateApiTokenResponseData(BaseModel):
    token: str
    uuid: str


class CreateApiTokenResponseDto(CreateApiTokenResponseData):
    pass


class DeleteApiTokenResponseDto(BaseModel):
    is_deleted: bool = Field(..., alias="isDeleted")


class ApiTokenDto(BaseModel):
    uuid: str
    token: str
    token_name: str = Field(..., alias="tokenName")
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")


class DocsInfoDto(BaseModel):
    is_docs_enabled: bool = Field(..., alias="isDocsEnabled")
    scalar_path: Optional[str] = Field(None, alias="scalarPath")
    swagger_path: Optional[str] = Field(None, alias="swaggerPath")


class FindAllApiTokensResponseData(BaseModel):
    api_keys: List[ApiTokenDto] = Field(..., alias="apiKeys")
    docs: DocsInfoDto


class FindAllApiTokensResponseDto(FindAllApiTokensResponseData):
    pass



import datetime
from typing import List
from uuid import UUID

from pydantic import BaseModel, Field, RootModel


class NodeUsageResponseDto(BaseModel):
    node_uuid: UUID = Field(alias="nodeUuid")
    node_name: str = Field(alias="nodeName")
    total: float
    total_download: float = Field(alias="totalDownload")
    total_upload: float = Field(alias="totalUpload")
    human_readable_total: str = Field(alias="humanReadableTotal")
    human_readable_total_download: str = Field(alias="humanReadableTotalDownload")
    human_readable_total_upload: str = Field(alias="humanReadableTotalUpload")
    date: datetime.date


class NodesUsageResponseDto(RootModel[List[NodeUsageResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class GetNodesUsageByRangeResponseDto(RootModel[List[NodeUsageResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class NodeRealtimeUsageResponseDto(BaseModel):
    node_uuid: UUID = Field(alias="nodeUuid")
    node_name: str = Field(alias="nodeName")
    country_code: str = Field(alias="countryCode")
    download_bytes: float = Field(alias="downloadBytes")
    upload_bytes: float = Field(alias="uploadBytes")
    total_bytes: float = Field(alias="totalBytes")
    download_speed_bps: float = Field(alias="downloadSpeedBps")
    upload_speed_bps: float = Field(alias="uploadSpeedBps")
    total_speed_bps: float = Field(alias="totalSpeedBps")


class NodesRealtimeUsageResponseDto(RootModel[List[NodeRealtimeUsageResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class GetNodesRealtimeUsageResponseDto(RootModel[List[NodeRealtimeUsageResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class UserUsageByRangeItem(BaseModel):
    user_uuid: UUID = Field(alias="userUuid")
    node_uuid: UUID = Field(alias="nodeUuid")
    node_name: str = Field(alias="nodeName")
    total: float
    date: str


class GetUserUsageByRangeResponseDto(RootModel[List[UserUsageByRangeItem]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class NodeUserUsageItem(BaseModel):
    user_uuid: UUID = Field(alias="userUuid")
    username: str
    node_uuid: UUID = Field(alias="nodeUuid")
    total: float
    date: str


class GetNodeUserUsageByRangeResponseDto(RootModel[List[NodeUserUsageItem]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]



from typing import Annotated

from rapid_api_client import Path
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateNodeRequestDto,
    CreateNodeResponseDto,
    DeleteNodeResponseDto,
    DisableNodeResponseDto,
    EnableNodeResponseDto,
    GetAllNodesResponseDto,
    GetOneNodeResponseDto,
    ReorderNodeRequestDto,
    ReorderNodeResponseDto,
    RestartAllNodesResponseDto,
    RestartNodeResponseDto,
    UpdateNodeRequestDto,
    UpdateNodeResponseDto,
    RestartAllNodesRequestDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class NodesController(BaseController):
    @post("/nodes", response_class=CreateNodeResponseDto)
    async def create_node(
        self,
        body: Annotated[CreateNodeRequestDto, PydanticBody()],
    ) -> CreateNodeResponseDto:
        """Create Node"""
        ...

    @get("/nodes", response_class=GetAllNodesResponseDto)
    async def get_all_nodes(
        self,
    ) -> GetAllNodesResponseDto:
        """Get All Nodes"""
        ...

    @get("/nodes/{uuid}", response_class=GetOneNodeResponseDto)
    async def get_one_node(
        self,
        uuid: Annotated[str, Path(description="Node UUID")],
    ) -> GetOneNodeResponseDto:
        """Get One Node"""
        ...

    @delete("/nodes/{uuid}", response_class=DeleteNodeResponseDto)
    async def delete_node(
        self,
        uuid: Annotated[str, Path(description="Node UUID")],
    ) -> DeleteNodeResponseDto:
        """Delete Node"""
        ...

    @patch("/nodes", response_class=UpdateNodeResponseDto)
    async def update_node(
        self,
        body: Annotated[UpdateNodeRequestDto, PydanticBody()],
    ) -> UpdateNodeResponseDto:
        """Update Node"""
        ...

    @post("/nodes/{uuid}/actions/enable", response_class=EnableNodeResponseDto)
    async def enable_node(
        self,
        uuid: Annotated[str, Path(description="Node UUID")],
    ) -> EnableNodeResponseDto:
        """Enable Node"""
        ...

    @post("/nodes/{uuid}/actions/disable", response_class=DisableNodeResponseDto)
    async def disable_node(
        self,
        uuid: Annotated[str, Path(description="Node UUID")],
    ) -> DisableNodeResponseDto:
        """Disable Node"""
        ...

    @post("/nodes/{uuid}/actions/restart", response_class=RestartNodeResponseDto)
    async def restart_node(
        self,
        uuid: Annotated[str, Path(description="Node UUID")],
    ) -> RestartNodeResponseDto:
        """Restart Node"""
        ...

    @post("/nodes/actions/restart-all", response_class=RestartAllNodesResponseDto)
    async def restart_all_nodes(
        self,
        body: Annotated[RestartAllNodesRequestDto, PydanticBody()],
    ) -> RestartAllNodesResponseDto:
        """Restart All Nodes"""
        ...

    @post("/nodes/actions/reorder", response_class=ReorderNodeResponseDto)
    async def reorder_nodes(
        self,
        body: Annotated[ReorderNodeRequestDto, PydanticBody()],
    ) -> ReorderNodeResponseDto:
        """Reorder Nodes"""
        ...



from datetime import datetime

import httpx

from remnawave.enums import ErrorCode
from .general import (
    ApiError,
    ApiErrorResponse,
    BadRequestError,
    ConflictError,
    ForbiddenError,
    NotFoundError,
    ServerError,
    UnauthorizedError,
)

ERRORS: dict[str, dict] = {
    ErrorCode.INTERNAL_SERVER_ERROR: ServerError,
    ErrorCode.LOGIN_ERROR: ServerError,
    ErrorCode.UNAUTHORIZED: UnauthorizedError,
    ErrorCode.FORBIDDEN_ROLE_ERROR: ForbiddenError,
    ErrorCode.CREATE_API_TOKEN_ERROR: ServerError,
    ErrorCode.DELETE_API_TOKEN_ERROR: ServerError,
    ErrorCode.REQUESTED_TOKEN_NOT_FOUND: NotFoundError,
    ErrorCode.FIND_ALL_API_TOKENS_ERROR: ServerError,
    ErrorCode.GET_PUBLIC_KEY_ERROR: ServerError,
    ErrorCode.ENABLE_NODE_ERROR: ServerError,
    ErrorCode.NODE_NOT_FOUND: NotFoundError,
    ErrorCode.CONFIG_NOT_FOUND: NotFoundError,
    ErrorCode.UPDATE_CONFIG_ERROR: ServerError,
    ErrorCode.GET_CONFIG_ERROR: ServerError,
    ErrorCode.DELETE_MANY_INBOUNDS_ERROR: ServerError,
    ErrorCode.CREATE_MANY_INBOUNDS_ERROR: ServerError,
    ErrorCode.FIND_ALL_INBOUNDS_ERROR: ServerError,
    ErrorCode.CREATE_USER_ERROR: ServerError,
    ErrorCode.USER_USERNAME_ALREADY_EXISTS: BadRequestError,
    ErrorCode.USER_SHORT_UUID_ALREADY_EXISTS: BadRequestError,
    ErrorCode.USER_SUBSCRIPTION_UUID_ALREADY_EXISTS: BadRequestError,
    ErrorCode.CREATE_USER_WITH_INBOUNDS_ERROR: ServerError,
    ErrorCode.CANT_GET_CREATED_USER_WITH_INBOUNDS: ServerError,
    ErrorCode.GET_ALL_USERS_ERROR: ServerError,
    ErrorCode.USER_NOT_FOUND: NotFoundError,
    ErrorCode.GET_USER_BY_ERROR: ServerError,
    ErrorCode.REVOKE_USER_SUBSCRIPTION_ERROR: ServerError,
    ErrorCode.DISABLE_USER_ERROR: ServerError,
    ErrorCode.USER_ALREADY_DISABLED: BadRequestError,
    ErrorCode.USER_ALREADY_ENABLED: BadRequestError,
    ErrorCode.ENABLE_USER_ERROR: ServerError,
    ErrorCode.CREATE_NODE_ERROR: ServerError,
    ErrorCode.NODE_NAME_ALREADY_EXISTS: BadRequestError,
    ErrorCode.NODE_ADDRESS_ALREADY_EXISTS: BadRequestError,
    ErrorCode.NODE_ERROR_WITH_MSG: ServerError,
    ErrorCode.NODE_ERROR_500_WITH_MSG: ServerError,
    ErrorCode.RESTART_NODE_ERROR: ServerError,
    ErrorCode.GET_CONFIG_WITH_USERS_ERROR: ServerError,
    ErrorCode.DELETE_USER_ERROR: ServerError,
    ErrorCode.UPDATE_NODE_ERROR: ServerError,
    ErrorCode.UPDATE_USER_ERROR: ServerError,
    ErrorCode.INCREMENT_USED_TRAFFIC_ERROR: ServerError,
    ErrorCode.GET_ALL_NODES_ERROR: ServerError,
    ErrorCode.GET_ONE_NODE_ERROR: ServerError,
    ErrorCode.DELETE_NODE_ERROR: ServerError,
    ErrorCode.CREATE_HOST_ERROR: ServerError,
    ErrorCode.HOST_REMARK_ALREADY_EXISTS: BadRequestError,
    ErrorCode.HOST_NOT_FOUND: NotFoundError,
    ErrorCode.DELETE_HOST_ERROR: ServerError,
    ErrorCode.GET_USER_STATS_ERROR: ServerError,
    ErrorCode.UPDATE_USER_WITH_INBOUNDS_ERROR: ServerError,
    ErrorCode.GET_ALL_HOSTS_ERROR: ServerError,
    ErrorCode.REORDER_HOSTS_ERROR: ServerError,
    ErrorCode.UPDATE_HOST_ERROR: ServerError,
    ErrorCode.CREATE_CONFIG_ERROR: ServerError,
    ErrorCode.ENABLED_NODES_NOT_FOUND: ConflictError,
    ErrorCode.GET_NODES_USAGE_BY_RANGE_ERROR: ServerError,
    ErrorCode.RESET_USER_TRAFFIC_ERROR: ServerError,
    ErrorCode.REORDER_NODES_ERROR: ServerError,
    ErrorCode.GET_ALL_INBOUNDS_ERROR: ServerError,
    ErrorCode.BULK_DELETE_USERS_BY_STATUS_ERROR: ServerError,
    ErrorCode.UPDATE_INBOUND_ERROR: ServerError,
    ErrorCode.CONFIG_VALIDATION_ERROR: ServerError,
    ErrorCode.USERS_NOT_FOUND: NotFoundError,
    ErrorCode.GET_USER_BY_UNIQUE_FIELDS_NOT_FOUND: NotFoundError,
    ErrorCode.UPDATE_EXCEEDED_TRAFFIC_USERS_ERROR: ServerError,
    ErrorCode.ADMIN_NOT_FOUND: NotFoundError,
    ErrorCode.CREATE_ADMIN_ERROR: ServerError,
    ErrorCode.GET_AUTH_STATUS_ERROR: ServerError,
    ErrorCode.FORBIDDEN_ONE: ForbiddenError,
    ErrorCode.DISABLE_NODE_ERROR: ServerError,
    ErrorCode.GET_ONE_HOST_ERROR: ServerError,
    ErrorCode.SUBSCRIPTION_SETTINGS_NOT_FOUND: NotFoundError,
    ErrorCode.GET_SUBSCRIPTION_SETTINGS_ERROR: ServerError,
    ErrorCode.UPDATE_SUBSCRIPTION_SETTINGS_ERROR: ServerError,
}


def handle_api_error(response: httpx.Response) -> None:
    if response.status_code >= 400:
        try:
            error_data = response.json()
            error_response = ApiErrorResponse(**error_data)
            
            # Fill missing fields for API v2 format
            if error_response.timestamp is None:
                error_response.timestamp = datetime.now()
            if error_response.path is None:
                error_response.path = response.request.url.path
            if error_response.code is None:
                # Use status_code or default to UNKNOWN
                if error_response.status_code:
                    error_response.code = f"HTTP_{error_response.status_code}"
                else:
                    error_response.code = "UNKNOWN"

            if error_response.code in ERRORS:
                exception_class = ERRORS[error_response.code]
            else:
                if response.status_code == 400:
                    exception_class = BadRequestError
                elif response.status_code == 401:
                    exception_class = UnauthorizedError
                elif response.status_code == 403:
                    exception_class = ForbiddenError
                elif response.status_code == 404:
                    exception_class = NotFoundError
                elif response.status_code == 409:
                    exception_class = ConflictError
                elif response.status_code >= 500:
                    exception_class = ServerError
                else:
                    exception_class = ApiError

            raise exception_class(response.status_code, error_response)
        except ValueError:
            raise ApiError(
                response.status_code,
                ApiErrorResponse(
                    timestamp=datetime.now(),
                    path=response.request.url.path,
                    message="Unknown error " + response.text,
                    code="UNKNOWN",
                ),
            )



from typing import Any, Dict

from pydantic import BaseModel


class ConfigData(BaseModel):
    config: Any


class GetConfigResponseDto(ConfigData):
    pass


class UpdateConfigRequestDto(BaseModel):
    pass  # Empty model as per OpenAPI spec


class UpdateConfigResponseDto(ConfigData):
    pass


# Legacy alias for backward compatibility
ConfigResponseDto = ConfigData



from .api_tokens_management import APITokensManagementController
from .auth import AuthController
from .bandwidthstats import BandWidthStatsController
from .config_profiles import ConfigProfilesController
from .hosts import HostsController
from .hosts_bulk_actions import HostsBulkActionsController
from .hwid import HWIDUserController
from .inbounds import InboundsController
from .inbounds_bulk_actions import InboundsBulkActionsController
from .infra_billing import InfraBillingController
from .internal_squads import InternalSquadsController
from .keygen import KeygenController
from .nodes import NodesController
from .nodes_usage_history import NodesUsageHistoryController, NodesUserUsageHistoryController
from .subscription import SubscriptionController
from .subscriptions_controller import SubscriptionsController
from .subscriptions_settings import SubscriptionsSettingsController
from .subscriptions_template import SubscriptionsTemplateController
from .system import SystemController
from .users import UsersController
from .users_bulk_actions import UsersBulkActionsController
from .users_stats import UsersStatsController
from .webhooks import WebhookUtility
from .xray_config import XrayConfigController
from .subscriptions_request import SubscriptionRequestHistoryController

__all__ = [
    "APITokensManagementController",
    "AuthController",
    "BandWidthStatsController",
    "ConfigProfilesController",
    "HostsController",
    "HostsBulkActionsController",
    "HWIDUserController",
    "InboundsController",
    "InboundsBulkActionsController",
    "InfraBillingController",
    "InternalSquadsController",
    "KeygenController",
    "NodesController",
    "NodesUsageHistoryController",
    "NodesUserUsageHistoryController",
    "SubscriptionController",
    "SubscriptionsController",
    "SubscriptionsSettingsController",
    "SubscriptionsTemplateController",
    "SystemController",
    "UsersController",
    "UsersBulkActionsController",
    "UsersStatsController",
    "WebhookUtility",
    "XrayConfigController",
    "SubscriptionRequestHistoryController",
]
